/* global goog, nsVmap, oVmap, ol, angular, bootbox */

/**
 * @author: Ludivine Premont
 * @Description: Fichier contenant la classe nsVmap.nsToolsManager.nsModules.Anomalies
 * cette classe permet l'initialisation du module Anomalies
 */
goog.provide('nsVmap.nsToolsManager.nsModules.Anomalies');

goog.require('oVmap');
goog.require('ol.Feature');
goog.require('ol.source.Vector');
goog.require('ol.source.Cluster');
goog.require('ol.style.Style');
goog.require('ol.style.Fill');
goog.require('ol.style.Stroke');
goog.require('ol.style.Circle');
goog.require('ol.style.Icon');

/**
 * @classdesc
 * Class {@link nsVmap.nsToolsManager.nsModules.Anomalies}: Add the Anomalies tools defined in data/tools.json
 * @param {object} opt_options options
 * @constructor
 * @export
 */
nsVmap.nsToolsManager.nsModules.Anomalies = function (opt_options) {
  oVmap.log('nsVmap.nsToolsManager.nsModules.Anomalies');
};
goog.exportProperty(nsVmap.nsToolsManager.nsModules, 'Anomalies', nsVmap.nsToolsManager.nsModules.Anomalies);


/**
 * appUiGridActionAnomalies directive.
 * Boutons de la colonne "Actions" du module ui-grid pour le module Anomalies.
 * @ngInject
 **/
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.appUiGridActionAnomaliesDrtv = function () {
  return {
    replace: true,
    templateUrl: oVmap['properties']['vmap_folder'] + '/' + 'template/vitis/uiGridActionAnomaliesTpl.html'
  };
};
oVmap.module.directive('appUiGridActionAnomalies', nsVmap.nsToolsManager.nsModules.Anomalies.prototype.appUiGridActionAnomaliesDrtv);

/**
 * App-specific directive wrapping the anomalies tools. The directive's
 * controller has a property "map" including a reference to the OpenLayers
 * map.
 *
 * @return {angular.Directive} The directive specs.
 * @constructor
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesDirective = function () {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesDirective");
  return {
    restrict: 'C',
    scope: {
      'map': '=appMap',
      'lang': '=appLang',
      'selection': '=appInfos',
      'currentAction': '=appAction'
    },
    controller: 'AppAnomaliesController',
    controllerAs: 'ctrl',
    bindToController: true,
    templateUrl: oVmap['properties']['vmap_folder'] + '/' + 'template/modules/anomalies.html'
  };
};


/**
 * Anomalies controller
 * @param {object} $scope
 * @export
 * @constructor
 * @ngInject
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController = function ($scope, $q, $translate) {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController");

  this.$q_ = $q;
  var this_ = this;

  this.properties = oVmap.properties;
  this.token = oVmap.properties.token;
  this.$scope_ = $scope;
  this.$translate = $translate;
  this.anomaliesInitialized = false;

  var aUserPrivileges = sessionStorage['privileges'].split(',');
  this.currentUserId = sessionStorage['user_id'];
  this.bHaveAdminPrivilege = false;
  this.bHaveUserPrivilege = false;
  this.bHaveLightUserPrivilege = false;
  if (goog.isArray(aUserPrivileges)) {
    if (aUserPrivileges.indexOf('vmap_anomalies_admin') !== -1) {
      this.bHaveAdminPrivilege = true;
    }
    if (aUserPrivileges.indexOf('vmap_anomalies_user') !== -1) {
      this.bHaveUserPrivilege = true;
    }
    if (aUserPrivileges.indexOf('vmap_anomalies_light_user') !== -1 && !this.bHaveUserPrivilege && !this.bHaveAdminPrivilege) {
      this.bHaveLightUserPrivilege = true;
    }
  }

  // Filter form model
  this.initValueFilterForm = {
    selectedStatus: [1,2,3], // Par défaut tout les statuts sauf Annulé et Terminé
    onlyMyAnomalies: false
  };
  this.filterForm = angular.copy(this.initValueFilterForm);

  // Cartographie des anomalies
  this.isDisplayAnomaliesOnMap = true;

  // Liste des anomalies
  this.anomalies = [];

  var opt_options = {};
  opt_options.style = function () {
    return this_.createDefaultStyle('#FE0101', '#FFFFFF');
  };
  
  /**
   * @type {ol.layer.Vector}
   * @private
   */
  this.oFormOverlayLayer_ = oVmap.getMap().addVectorLayer(opt_options);

  /**
   * @type {ol.Collection}
   * @private
   */
  this.oFormOverlayFeatures_ = this.oFormOverlayLayer_.getSource().getFeaturesCollection();

  $scope.featuresLength = 0;
  this.oFormOverlayFeatures_.on('change:length', function () {
    $scope.$applyAsync(function () {
      $scope.featuresLength = this_.oFormOverlayFeatures_.getLength();
    });
  }, this);

  /**
   * @type {ol.interaction.Select}
   * @private
   */
  this.delete_ = new ol.interaction.Select({
    layers: [this.oFormOverlayLayer_]
  });

  // Supprime la feature lors de la sélection sur l'interaction this.delete_
  this.delete_.on('select', function () {

    // Supprime la partie selectionnée
    var aSelectedFeatures = this.delete_.getFeatures().getArray();
    for (var i = 0; i < aSelectedFeatures.length; i++) {
      this.oFormOverlayLayer_.getSource().removeFeature(aSelectedFeatures[i]);
    }
    this.delete_.getFeatures().clear();

    // Arrête la suppression si il ne reste plus de parties
    if (goog.isDefAndNotNull(this.oFormOverlayLayer_.getSource().getFeatures().length)) {
      if (this.oFormOverlayLayer_.getSource().getFeatures().length === 0) {
        oVmap.getMap().removeActionsOnMap();
        oVmap.getMap().getMapTooltip().hide();
      }
    }

  }, this);

  this.drag_ = new ol.interaction.Drag();

  // Lance updateGeomFeature_ lors de chaque changement sur this.oFormOverlayFeatures_
  // si il n'y a pas eut de changements pendant 500ms
  var iTmpChanges = 0;
  this.oFormOverlayLayer_.getSource().on('change', function () {
    iTmpChanges++;
    var iTmpChangesCopy = angular.copy(iTmpChanges);
    setTimeout(function () {
      if (iTmpChanges === iTmpChangesCopy) {

        var EWKTGeometry = this_.updateGeomFeature_();

        // Écrit la géométrie WKT
        // Si la palette est visible c'est une édition
        if (!$('#anomalies-modify-geom-palette').is(':visible') && goog.isDefAndNotNull(EWKTGeometry)) {
          var insertFormScope = this_.getAnomaliesFormReaderScope('vmap_anomalies_insert_form');
          // essaye d'initialiser le formulaire
          var oFormValues = {
            insert: {}
          };
          this_.initForm('vmap_anomalies_insert_form', 'insert', oFormValues).then(function () {
            insertFormScope.oFormValues[insertFormScope.sFormDefinitionName].geom = EWKTGeometry;
          });
        }
      }
    }, 500);
  }, this);

  this.initTooltip('anomalies-insert-locate-i', {
    placement: 'top',
    content: 'ANOMALIES_LOCATE_BTN_TOOLTIP'
  });

  // Annule l'affichage des formulaires quand on change d'outil
  oVmap['scope'].$on('toggleOutTools', function () {
    // Efface les couches de formulaire
    if (goog.isDefAndNotNull(this_.oFormOverlayLayer_)) {
      this_.oFormOverlayLayer_.getSource().clear();
    }
  });

  // Prépare le style pour les features
  this.getStyles().then(function (styles) {
    this_.styles = styles;
  });
  var opt_options = {};
  opt_options.style = function (feature) {
    return this_.getStyle(feature, this_);
  };

  /**
   * @type {ol.layer.Vector}
   * @private
   */
  this.oOverlayLayer_ = oVmap.getMap().addVectorLayer(opt_options);


  oVmap['scope'].$on('moduleChanged', function (event, opt) {
    if (opt['moduleId'] != opt['openedTool'] && opt['moduleId'] == 1) {
      // Active par défaut le second onglet (pour les utilisateurs léger)
      if (opt['openedTool'] == '_' && this_.bHaveLightUserPrivilege === true && this_.anomaliesInitialized === false) {
        this_.initAnomalies();
      } else {
        // Rafraichi la liste des anomalies et les affichent sur la carte
        this_.getAnomalies();
      }
    } else {
      // Cache les anomalies
      this_.oOverlayLayer_.getSource().clear();
    }
  });

  // init la grid
  this.initUiGrid();
}

/**
 * Gère les points sur la carte
 * @param {*} type insert/moveFeature/deleteFeature
 * @param {*} isActive
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.startInsertion = function (type, isActive) {
  oVmap.log('nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.startInsertion');

  // Supprime les interractions
  oVmap.getMap().removeActionsOnMap();

  // Cache la tooltip
  oVmap.getMap().getMapTooltip().hide();

  if (isActive === true)
    return 0;

  var message = '';
  // insert
  if (type === 'insert') {
    message = 'Cliquez pour commencer à dessiner';
    // Ajoute l'interraction
    oVmap.getMap().setDrawInteraction({
      type: 'Point',
      features: this.oFormOverlayFeatures_
    }, 'anomalies-insertPoint');

  } else if (type === 'moveFeature') {
    message = 'Cliquez sur la géométrie et rester appuyé pour la déplacer';
    // Ajoute l'interraction
    oVmap.getMap().setInteraction(this.drag_, 'anomalies-movePoint');

  } else if (type === 'deleteFeature') {
    message = 'Cliquez sur une partie pour la supprimer';
    // Ajoute l'interraction
    oVmap.getMap().setInteraction(this.delete_, 'anomalies-deletePoint');

  } else {
    console.error('startSelection: type (' + type + ') not allowed');
    return 0;
  }

  // Ajoute la tooltip
  oVmap.getMap().getMapTooltip().displayMessage(message);

};

/**
 * Met à jour le champs geom du formulaire
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.updateGeomFeature_ = function() {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.updateGeomFeature_");

  var aOverlayFeatures = goog.array.clone(this.oFormOverlayFeatures_.getArray());

  // Transforme la feature en WKT
  var WKT = new ol.format.WKT();
  var WKTGeometry = '';

  if (aOverlayFeatures.length > 0) {
    var oMultiPoint = new ol.geom.MultiPoint();
    for (var i = 0; i < aOverlayFeatures.length; i++) {
      if (aOverlayFeatures[i].getGeometry().getType() === 'Point') {
        oMultiPoint.appendPoint(aOverlayFeatures[i].getGeometry());
      } else if (aOverlayFeatures[i].getGeometry().getType() === 'MultiPoint') {
        oMultiPoint.appendPoint(aOverlayFeatures[i].getGeometry());
      }
    }
    var Feature = new ol.Feature({
      geometry: oMultiPoint
    });

    WKTGeometry = WKT.writeFeature(Feature, {
      dataProjection: this['map'].getView().getProjection().getCode(),
      featureProjection: this['map'].getView().getProjection().getCode()
    });
  }

  if (WKTGeometry === '') {
    return null;
  }

  // Transforme la feature en EWKT
  var EWKTGeometry = oVmap.getEWKTFromWKT(WKTGeometry, this['map'].getView().getProjection().getCode());

  return EWKTGeometry;
}

/**
 * Initialise un formulaire d'anomalie
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.initForm = function (id, definitionName, oFormValues) {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.initForm");
  var this_ = this;
  var deferred = this.$q_.defer();
  var formScope = this.getAnomaliesFormReaderScope(id);

  if (!goog.isDefAndNotNull(formScope.oFormDefinition) || angular.equals(formScope.oFormDefinition, {}) ||
    angular.equals(formScope.oFormDefinition[formScope.sFormDefinitionName], {})) {
    this.getForm_().then(function (oFormDefinition) {
      oFormValues[definitionName].bHaveAdminPrivilege = this_.bHaveAdminPrivilege;
      oFormValues[definitionName].bHaveUserPrivilege = this_.bHaveUserPrivilege;
      oFormValues[definitionName].bHaveLightUserPrivilege = this_.bHaveLightUserPrivilege;

      // Affiche le formulaire
      formScope['ctrl']['setDefinitionName'](definitionName);
      formScope['ctrl']['setFormValues'](oFormValues);
      formScope['ctrl']['setFormDefinition'](oFormDefinition);
      formScope.$evalAsync(function () {
        formScope['ctrl']['loadForm']();
        deferred.resolve();
      });
    });
  } else if (goog.isDefAndNotNull(oFormValues[definitionName]) && !angular.equals(oFormValues[definitionName], {}) ) {
    oFormValues[definitionName].bHaveAdminPrivilege = this.bHaveAdminPrivilege;
    oFormValues[definitionName].bHaveUserPrivilege = this.bHaveUserPrivilege;
    oFormValues[definitionName].bHaveLightUserPrivilege = this.bHaveLightUserPrivilege;

    formScope['ctrl']['setFormValues'](oFormValues);
    formScope.$evalAsync(function () {
      formScope['ctrl']['loadForm']();
      deferred.resolve();
    });
  } else {
    deferred.resolve();
  }
  return deferred.promise;
}

/**
 * Get the anomaly form in a promise
 * @returns {defer.promise}
 * @private
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.getForm_ = function () {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.getForm_");

  var deferred = this.$q_.defer();

  ajaxRequest({
    'method': 'GET',
    'url': 'modules/vmap/forms/anomalies/anomalies.json',
    'headers': {
      'Accept': 'application/x-vm-json'
    },
    'ajaxLoader': false,
    'scope': this.$scope_,
    'success': function (response) {
      if (goog.isDefAndNotNull(response['data'])) {
        deferred.resolve(response['data']);
      }
    }
  });
  return deferred.promise;
};

nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.insertAnomaly = function () {
  oVmap.log('nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.insertAnomaly');

  var insertFormScope = this.getAnomaliesFormReaderScope('vmap_anomalies_insert_form');
  // Set user id et map_id
  var oFormValues = insertFormScope.oFormValues[insertFormScope.sFormDefinitionName];
  oFormValues.user_id = sessionStorage['user_id'];
  var oMapCatalog = oVmap.getMapManager().getMapCatalog();
  oFormValues.map_id = oMapCatalog["maps"][oMapCatalog["usedMap"]]["map_id"];
  
  this.checkAndSendForm(insertFormScope);
}

/**
 * Lance la vérification des données puis affiche une erreur ou lance l'enregistrement du formulaire
 * @param {*} formScope 
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.checkAndSendForm = function (formScope) {
  oVmap.log('nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.checkAndSendForm');
  var deferred = this.$q_.defer();
  // Check Form values
  var formIsValid = this.formIsValid(formScope);
  if (formIsValid === true) {
    // Get formData and set hidden values
    var oFormData = this.getFormData(formScope);

    // Send form
    this.sendForm(oFormData, formScope.sFormDefinitionName).then(function(result) {
      deferred.resolve(result);
    });
  } else {
    // Print error
    if (goog.isArray(formIsValid)) {
      var alertMessage = "Votre requête n'a pas pu aboutir, les attributs suivants sont obligatoires: <br>";
      for (var i = 0; i < formIsValid.length; i++) {
        alertMessage = (i === 0 ? alertMessage : alertMessage + ', ');
        alertMessage += this.$translate.instant(formIsValid[i]);
      }
    } else {
      var alertMessage = "Votre requête n'a pas pu aboutir, certains attributs obligatoires n'ont pas étés renseignés";
    }

    bootbox['dialog']({
      'message': alertMessage,
      'buttons': {
        'cancel': {
          'label': "Annuler",
          'className': "btn-default"
        }
      }
    });
    deferred.reject(false);
  }

  return deferred.promise;
}

/**
 * Récupère les données du formulaire
 * @param {*} formScope 
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.getFormData = function (formScope) {
  oVmap.log('nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.getFormData');

  // Prepare env
  var envSrvc = angular.element(vitisApp.appMainDrtv).injector().get(["envSrvc"]);
  envSrvc.oFormValues = formScope.oFormValues;
  envSrvc.oFormDefinition = formScope.oFormDefinition;
  envSrvc.sFormDefinitionName = formScope.sFormDefinitionName;
  // GetFormData
  var formSrvc = angular.element(vitisApp.appMainDrtv).injector().get(["formSrvc"]);
  
  var oFormData = formSrvc['getFormData'](formScope.sFormDefinitionName, false);
  if (goog.isDefAndNotNull(formScope.oFormValues[sFormDefinitionName].geom)) {
    oFormData.append('geom', formScope.oFormValues[sFormDefinitionName].geom);
  }

  return oFormData;
}

/**
 * Envoi le formulaire au webservice
 * @param {*} oFormData 
 * @param {*} sFormDefinitionName 
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.sendForm = function (oFormData, sFormDefinitionName) {
  oVmap.log('nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.sendForm');

  var this_ = this
  var propertiesSrvc = angular.element(vitisApp.appMainDrtv).injector().get(['propertiesSrvc']);
  var envSrvc = angular.element(vitisApp.appMainDrtv).injector().get(["envSrvc"]);
  var deferred = this.$q_.defer();

  // Paramètres de la requête.
  var sRequestUrl = propertiesSrvc['web_server_name'] + '/' + propertiesSrvc['services_alias'] + '/anomalies/anomalies';
  var sRequestMethod;
  if (sFormDefinitionName == 'update') {
    sRequestMethod = 'PUT';
    sRequestUrl += '/' + envSrvc.oFormValues[envSrvc.sFormDefinitionName].anomalies_id;
  } else {
    sRequestMethod = 'POST';
  }

  // Requête Ajax de création ou mise à jour d'un enregistrement.
  ajaxRequest({
    'method': sRequestMethod,
    'url': sRequestUrl,
    'data': oFormData,
    "scope": this.$scope_,
    'success': function (response) {
      //
      if (response["data"]["status"] == 1) {
        // Affichage du message de succés.
        this_.$translate("FORM_VALIDATION_OK").then(function (sTranslation) {
          this_.resetForm();
          $.notify(sTranslation, "success");
          deferred.resolve(true);
          // Rafraichie les anomalies
          this_.getAnomalies();
        });
      }
      else {
        deferred.resolve(false);

        var sText = '';
        // Message d'erreur ?
        if (typeof (response["data"]["errorMessage"]) != "undefined" && response["data"]["errorMessage"] != null) {
          sText = response["data"]["errorMessage"];
        }

        // Paramètres de la fenêtre modale.
        var oOptions = {
          "className": "modal-danger",
          "message": sText
        };
        // Affichage de la fenêtre modale.
        this_.$scope_.$root["modalWindow"]("dialog", "FORM_VALIDATION_ERROR", oOptions);
      }
    }
  });
  return deferred.promise;
}

/**
 * Vérifie si le formulaire est valide
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.formIsValid = function (formScope) {
  oVmap.log('nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.formIsValid');

  var aUnregistredFields = [];
  var rows = formScope.oFormDefinition[formScope.sFormDefinitionName].rows;
  var oFormValues = formScope.oFormValues[formScope.sFormDefinitionName];

  for (var i = 0; i < rows.length; i++) {
    for (var ii = 0; ii < rows[i].fields.length; ii++) {
      if (rows[i].fields[ii].required === true) {
        var name = rows[i].fields[ii].name;
        var isSelectNotValid = (goog.isDefAndNotNull(oFormValues[name]) && goog.isDefAndNotNull(oFormValues[name]['options']) &&
          (!goog.isDefAndNotNull(oFormValues[name]['selectedOption']) || oFormValues[name]['selectedOption'].value === ''));

        if ((!goog.isDefAndNotNull(oFormValues[name]) || oFormValues[name] === "") || isSelectNotValid) {

          // Si le label est renseigné, communique le label, sinon le nom
          if (goog.isString(rows[i].fields[ii].label) && rows[i].fields[ii].label !== "")
            aUnregistredFields.push(rows[i].fields[ii].label);
          else
            aUnregistredFields.push(name);
        }
      }
    }
  }

  if (aUnregistredFields.length === 0)
    return true;
  else
    return aUnregistredFields;
}

/**
 * Reset form
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.resetForm = function () {
  var insertFormScope = this.getAnomaliesFormReaderScope('vmap_anomalies_insert_form');
  insertFormScope.oFormDefinition = {};
  insertFormScope.oFormValues[insertFormScope.sFormDefinitionName] = {};

  // Supprime les actions sur la carte
  oVmap.getMap().removeActionsAndTooltips();
  // Ferme les autres outils
  oVmap.getToolsManager().getBasicTools().toggleOutTools();
};

/**
 * Get the anomalies FormReader scope
 * @returns {angular.scope}
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.getAnomaliesFormReaderScope = function (id) {
  return angular.element($('#' + id).children()).scope();
};

/**
 * Get styles in a promise
 * @returns {defer.promise}
 * @private
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.getStyles = function () {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.getStyles");

  var propertiesSrvc = angular.element(vitisApp.appMainDrtv).injector().get(['propertiesSrvc']);
  var deferred = this.$q_.defer();

  ajaxRequest({
    'method': 'GET',
    'url': propertiesSrvc['web_server_name'] + '/' + propertiesSrvc['services_alias'] + '/anomalies/anomaliesfeaturestyles',
    'headers': {
      'Accept': 'application/x-vm-json'
    },
    'scope': this.$scope_,
    'success': function (response) {
      if (goog.isDefAndNotNull(response['data'])) {
        deferred.resolve(response['data']['data']);
      }
    }
  });
  return deferred.promise;
};

/**
 * Récupère le style en fonction des properties pour l'affichage des couches
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.getStyle = function (feature, this_) {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.getStyle");
  var status = feature.attributes.status_name;
  var theme = feature.attributes.theme_name;

  var statusThemeStyle;
  var statusStyle;
  var themeStyle;
  var defaultStyle;

  if (goog.isDefAndNotNull(this_.styles)) {
    for (var i = 0; i < this_.styles.length; i++) {
      if (goog.isDefAndNotNull(this_.styles[i].style)) {

        // Récupération du style par défaut (status_name et theme_name => NULL)
        if (!goog.isDefAndNotNull(this_.styles[i].status_name) && !goog.isDefAndNotNull(this_.styles[i].theme_name)) {
          defaultStyle = this_.styles[i].style;
        }

        // Si la feature rentre dans un cas, on sauvegarde le style
        if (this_.styles[i].status_name === status && this_.styles[i].theme_name === theme) {
          statusThemeStyle = this_.styles[i].style;
        } else if (this_.styles[i].status_name === status && !goog.isDefAndNotNull(this_.styles[i].theme_name)) {
          statusStyle = this_.styles[i].style;
        } else if (this_.styles[i].theme_name === theme && !goog.isDefAndNotNull(this_.styles[i].status_name)) {
          themeStyle = this_.styles[i].style;
        }
      }
    }

    // On applique le style en fonction de l'importance
    // Si theme et status correspondant > seulement status correspond > seulement theme correspond > style par défaut de la base
    if (goog.isDefAndNotNull(statusThemeStyle)) {
      feature.setStyleByJSON(statusThemeStyle);
    } else if (goog.isDefAndNotNull(statusStyle)) {
      feature.setStyleByJSON(statusStyle);
    } else if (goog.isDefAndNotNull(themeStyle)) {
      feature.setStyleByJSON(themeStyle);
    } else if (goog.isDefAndNotNull(defaultStyle)) {
      feature.setStyleByJSON(defaultStyle);
    } else {
      // Si aucun paramètre ne correspond on créer un style standard
      return this_.createDefaultStyle();
    }

    setTimeout(function() {
      this_.oOverlayLayer_.getSource().dispatchEvent('change');
    }, 50);
    return;
  } else {
    return this_.createDefaultStyle();
  }
}

/**
 * Créer un style de type picto
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.createDefaultStyle = function (outline, color) {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.createDefaultStyle");
  return new ol.style.Style({
    image: new ol.style.Circle({
      radius: 5,
      stroke: new ol.style.Stroke({
        color: color ? color : '#fff',
      }),
      fill: new ol.style.Fill({
        color: outline ? outline:'#3399CC',
      }),
    }),
  });
}

nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.initAddAnomaly = function () {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.initAddAnomaly");
  if (this.bHaveAdminPrivilege == true || this.bHaveUserPrivilege == true) {
    // Supprime les actions sur la carte
    oVmap.getMap().removeActionsAndTooltips();

    // Ferme les autres outils
    oVmap.getToolsManager().getBasicTools().toggleOutTools();
  }
}


nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.initAnomalies = function () {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.initAnomalies");
  var this_ = this;
  // Exécuté 1 seule fois
  if (this.anomaliesInitialized === false) {
    this.anomaliesInitialized = true;
    // init status list
    this.getAnomaliesStatus_();

    /**
     * @type {ol.interaction.Select}
     * @private
     */
    this.select_ = new ol.interaction.Select({
      layers: [this.oOverlayLayer_]
    });

    // Ouvre la fenetre de visualisation lors de la sélection sur l'interaction this.select_
    this.select_.on('select', function () {
      var aSelectedFeatures = this.select_.getFeatures().getArray();
      for (var i = 0; i < aSelectedFeatures.length; i++) { 
        this_.anomalySelected = aSelectedFeatures[i].attributes;
      }
      this_.openFormAnomaly(this_.anomalySelected, 'display');
    }, this);
  }

  // Executé a chaque fois

  // Supprime les actions sur la carte
  oVmap.getMap().removeActionsAndTooltips();  
  // Ferme les autres outils
  oVmap.getToolsManager().getBasicTools().toggleOutTools();

  // Init anomalies list
  this.getAnomalies();
}

/**
 * Get the anomalies status in a promise
 * @returns {defer.promise}
 * @private
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.getAnomaliesStatus_ = function () {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.getAnomaliesStatus_");

  var this_ = this;
  var deferred = this.$q_.defer();
  var propertiesSrvc = angular.element(vitisApp.appMainDrtv).injector().get(['propertiesSrvc']);

  ajaxRequest({
    'method': 'GET',
    'url': propertiesSrvc['web_server_name'] + '/' + propertiesSrvc['services_alias'] + '/anomalies/statuss',
    'params': {
      'order_by': 'status_id'
    },
    'headers': {
      'Accept': 'application/x-vm-json'
    },
    'scope': this.$scope_,
    'success': function (response) {
      if (goog.isDefAndNotNull(response.data) && response.data.status == 1) {
        // Set status list
        this_.statusList = response.data.data;
        
        deferred.resolve(response.data.data);
      }
    }
  });
  return deferred.promise;
};

/**
 * Get the anomalies in a promise
 * @returns {defer.promise}
 * @private
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.getAnomalies = function () {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.getAnomalies");

  var this_ = this;
  var deferred = this.$q_.defer();
  var propertiesSrvc = angular.element(vitisApp.appMainDrtv).injector().get(['propertiesSrvc']);

  var filters = this.getFilters_()
  ajaxRequest({
    'method': 'GET',
    'url': propertiesSrvc['web_server_name'] + '/' + propertiesSrvc['services_alias'] + '/anomalies/anomalies',
    'params': {
      'filter': filters,
      'order_by': 'anomalies_date',
      'sort_order': 'DESC'
    },
    'headers': {
      'Accept': 'application/x-vm-json'
    },
    'scope': this.$scope_,
    'success': function (response) {
      if (goog.isDefAndNotNull(response.data) && response.data.status == 1) {
        // Set anomalies list
        this_.anomalies = response.data.data;

        if (this_.anomalies) {
          for (var i = 0; i < this_.anomalies.length; i++) {
            var anomaly = this_.anomalies[i];
            if (this_.haveUpdateRight(anomaly) && this_.haveActiveStatut(anomaly)) {
              anomaly.edit_column = 'edit';
            }
          }
          this_.$scope_['gridOptions']['data'] = this_.anomalies;
        } else {
          this_.$scope_['gridOptions']['data'] = [];
        }

        // Relance l'affichage sur la carte si besoin
        this_.displayAnomaliesOnMap();
        
        // promise
        deferred.resolve(response.data.data);
      }
    }
  });
  return deferred.promise;
};

/**
 * Filtre la liste des anomalies
 * @returns {angular.scope}
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.getFilters_ = function () {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.getFilters_");

  var aFilters = [];
  if (goog.isDefAndNotNull(this.filterForm.selectedStatus) && goog.isArray(this.filterForm.selectedStatus)) {
    var values = [parseInt(this.filterForm.selectedStatus[0])];
    if (this.filterForm.selectedStatus.length > 1) {
      for (var i = 1; i < this.filterForm.selectedStatus.length; i++) {
        values.push(parseInt(this.filterForm.selectedStatus[i]));
      }
    }
    aFilters.push({
      "column": 'status_id',
      "compare_operator": 'IN',
      "value": values
    });
  }

  if (this.filterForm.onlyMyAnomalies == true) {
    aFilters.push({
      "column": 'user_id',
      "compare_operator": '=',
      "value": parseInt(sessionStorage['user_id'], 10)
    });
  }

  if (aFilters.length > 0) {
    filter = {
      "relation": "AND",
      "operators": []
    };
    for (var i = 0; i < aFilters.length; i++) {
      filter['operators'].push(aFilters[i]);
    }
  } else {
    filter = "";
  }

  return filter;
};

/**
 * Reset le formulaire de filtre
 * @returns {angular.scope}
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.resetFilterForm = function () {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.resetFilterForm");
  // Réinitalise les valeurs du formulaire de filtre
  this.filterForm = {
    selectedStatus: [1,2,3,4,5],
    onlyMyAnomalies: false
  };
  // Rafraichi la récupération des anomalies
  this.getAnomalies();
};

/**
 * Affiche ou cache les anomalies sur la carte
 * @returns {angular.scope}
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.displayAnomaliesOnMap = function () {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.displayAnomaliesOnMap");

  this.oOverlayLayer_.getSource().clear();

  if (this.isDisplayAnomaliesOnMap === true && goog.isArray(this.anomalies)) {
    // Affiche les points
    var proj = oVmap.getMap().getOLMap().getView().getProjection();
    for (var i = 0; i < this.anomalies.length; i++) {
      var oFeatures = ol.getFeaturesFromEWKT(this.anomalies[i].geom, proj);
      this.anomalies[i].geomFeature = oFeatures[0].getGeometry();
      oFeatures[0].attributes = this.anomalies[i];
      this.oOverlayLayer_.getSource().addFeature(oFeatures[0]);
    }
  }
};

/**
 * Permet de selectionner une anomalies sur la carte puis lance le formulaire display
 * @returns {angular.scope}
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.selectOnMap = function () {
  message = 'Cliquez sur une partie pour la selectionner';
  // Ajoute l'interraction
  oVmap.getMap().setInteraction(this.select_, 'anomalies-selectPoint');
};

/**
 * Affiche ou cache les anomalies sur la carte
 * @returns {angular.scope}
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.initTooltip = function (id, tooltip) {
  if (!goog.isDefAndNotNull(tooltip.trigger)) {
    tooltip.trigger = "hover";
  }
  this.$translate([tooltip.title, tooltip.content]).then(function (translations) {
    tooltip.title = translations[tooltip.title];
    tooltip.content = translations[tooltip.content];
    // Création du toolltip
    $('#'+ id)['popover']("destroy");
    setTimeout(function () {
      $('#'+ id)['popover'](tooltip)
        .on('inserted.bs.popover', function () {
          // La largeur est spécifiée ?
          if (typeof (tooltip.width) !== "undefined")
            $('#' + id).child(".popover").style.width = tooltip.width;
        });
    }, 200);
  });
};

nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.editFormFeature = function (anomaly, sFormDefinitionName) {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.editFormFeature");

  // Cache les anomalies
  this.oOverlayLayer_.getSource().clear();

  // Supprime les actions sur la carte
  oVmap.getMap().removeActionsAndTooltips();
  // Ferme les autres outils
  oVmap.getToolsManager().getBasicTools().toggleOutTools();

  this.sFormDefNameCurrentGeomEdition = 'update';

  // Ferme le formulaire d'édition
  $('#anomalies-edit-modal').modal('hide');
  // Ferme le formulaire de visualisation
  $('#anomalies-display-modal').modal('hide');

  // Lance l'édition
  $('#anomalies-modify-geom-palette').show();

  // Supprime les anomalies de la carte pour plus de clarté
  this.oOverlayLayer_.getSource().clear();
  // Ajoute la feature du formulaire
  var proj = oVmap.getMap().getOLMap().getView().getProjection();
  var oFeatures = ol.getFeaturesFromEWKT(anomaly.geom, proj);
  var points = oFeatures[0].getGeometry().getPoints();
  for (var i = 0; i < points.length; i++) {
    this.oFormOverlayLayer_.getSource().addFeature(new ol.Feature(points[i]));
  }

  if (sFormDefinitionName === 'display') {
    this.sFormDefNameCurrentGeomEdition = 'display';
    var displayFormScope = this.getAnomaliesFormReaderScope('anomalies_display_form_reader');
    var oFormValues = {
      update: displayFormScope.oFormValues.display
    }
    // Initialise le formulaire d'édition pour pouvoir editer la géométrie
    this.initForm('anomalies_edit_form_reader', 'update', oFormValues); 
  }
}

/**
 * Remove all was did during the geometry edition
 * @export
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.cancelEdition = function () {
  oVmap.log('nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.finishEdition');

  if (this.sFormDefNameCurrentGeomEdition === 'update') {
    this.finishEdition();
    // Affiche le formulaire d'édition
    $('#anomalies-edit-modal').modal('show');
  } else {
    this.finishEdition();
    // Affiche le formulaire d'édition
    $('#anomalies-display-modal').modal('show');
  }
};

/**
 * Remove all was did during the geometry edition
 * @export
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.finishEdition = function () {
  oVmap.log('nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.finishEdition');

  // Supprime la feature
  this.oFormOverlayLayer_.getSource().clear();

  // Réaffiche les anomalies si besoin
  this.displayAnomaliesOnMap();

  // cache la palette d'édition
  oVmap.getToolsManager().getBasicTools().toggleOutTools();

  this.sFormDefNameCurrentGeomEdition = null;
};

/**
 * Write the feature into the EditForm oFormValues and open it
 * @export
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.finishEditionAndDisplayForm = function () {
  oVmap.log('nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.finishEditionAndDisplayForm');
  var this_ = this;

  if (goog.isDefAndNotNull(this.sFormDefNameCurrentGeomEdition)) {
    var updateFormScope = this.getAnomaliesFormReaderScope('anomalies_edit_form_reader');
    var EWKTGeometry = this_.updateGeomFeature_();
    // Sauvegarde la géométrie
    updateFormScope.oFormValues[updateFormScope.sFormDefinitionName].geom = EWKTGeometry;

    if (this.sFormDefNameCurrentGeomEdition === 'display') {
      // Sauvegarde la valeur
      this.checkAndSendForm(updateFormScope).then(function (result) {
        if (result == true) {
          this_.getAnomalies(); // Rafraichie la liste des anomalies
          
          // Ferme l'édition de géométrie
          this_.finishEdition();

          // Affiche le formulaire d'affichage
          $('#anomalies-display-modal').modal('show');
        }
      });
    }

    if (this.sFormDefNameCurrentGeomEdition === 'update') {
      this_.finishEdition();

      // Affiche le formulaire d'édition
      $('#anomalies-edit-modal').modal('show');
    }
  }
};

/**
 * Action d'un bouton pour lancer la mise à jour du formulaire.
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.updateAnomaly = function (form) {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.updateAnomaly");
  var this_ = this;
  var deferred = this.$q_.defer();
  if (!goog.isDefAndNotNull(form)) {
    form = 'edit'
  }
  var updateFormScope = this.getAnomaliesFormReaderScope('anomalies_edit_form_reader');
  this.checkAndSendForm(updateFormScope).then(function(result) {
    if (result == true) {
      this_.getAnomalies(); // Rafraichie la liste des anomalies
      // Rafraichie le formulaire display
      var displayFormScope = this_.getAnomaliesFormReaderScope('anomalies_display_form_reader');
      displayFormScope.ctrl.setFormValues(updateFormScope.oFormValues);
      // Ferme le formulaire en cours
      $('#anomalies-' + form + '-modal').modal('hide');
    }
    deferred.resolve(result);
  });
  return deferred.promise;
}
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.updateStatusAnomaly = function (form, statusId) {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.updateStatusAnomaly");
  var this_ = this;
  var updateFormScope = this.getAnomaliesFormReaderScope('anomalies_edit_form_reader');

  // Si on n'est sur le formulaire display, on créer un formulaire update à partir de celui de display
  if (form === 'display') {
    var displayFormScope = this.getAnomaliesFormReaderScope('anomalies_display_form_reader');
    var oFormValues = {
      update: displayFormScope.oFormValues.display
    }
    this.initForm('anomalies_edit_form_reader', 'update', oFormValues);
  }

  statusId = {
    selectedOption: {
      value: statusId
    }
  };

  // Attend que le formulaire soit initialisé
  setTimeout(function() {
    var currentStatus = updateFormScope.oFormValues.update.status_id;
    updateFormScope.oFormValues.update.status_id = statusId;
    updateFormScope['ctrl'].setFormValues(updateFormScope.oFormValues);

    // Attend que le setFormValues est pris effet
    setTimeout(function() {
      this_.updateAnomaly(form).then(function (result) {
        // Si la modification n'a pas fonctionné remmet l'ancien status
        if (result != true) {
          var formScope = updateFormScope;
          if (form === 'display') {
            formScope = this_.getAnomaliesFormReaderScope('anomalies_display_form_reader');
          }
          
          formScope.oFormValues.update.status_id = currentStatus;
          formScope['ctrl'].setFormValues(formScope.oFormValues);
        }
      });
    }, 500);
  }, 500);
}

/**
 * ouvre le formulaire d'edition ou d'affichage d'anomaly
 * @param {any} anomalySelected? par défaut this.anomalySelected
 * @param {string} formType? par défaut update
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.openFormAnomaly = function (anomalySelected, formType) {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.openFormAnomaly");
  var this_ = this;
  if (!goog.isDefAndNotNull(anomalySelected)) {
    anomalySelected = this.anomalySelected;
  } else {
    this.anomalySelected = anomalySelected;
  }

  this.getAnomaly(anomalySelected.anomalies_id).then(function (oValues) {
    var oFormValues = {
      update: oValues,
      display: oValues
    };
  
    if (formType == 'display') {
      this_.initForm('anomalies_display_form_reader', 'display', oFormValues);
      $('#anomalies-display-modal').modal('show');
    } else {
      this_.initForm('anomalies_edit_form_reader', 'update', oFormValues);
      $('#anomalies-edit-modal').modal('show');
    }
  });

}

/**
 * Get a anomalie in a promise
 * @returns {defer.promise}
 * @private
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.getAnomaly = function (anomalyId) {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.getAnomalies");

  var deferred = this.$q_.defer();
  var propertiesSrvc = angular.element(vitisApp.appMainDrtv).injector().get(['propertiesSrvc']);

  if (!goog.isDefAndNotNull(anomalyId)) {
    anomalyId = this.anomalySelected.anomalies_id;
  }

  ajaxRequest({
    'method': 'GET',
    'url': propertiesSrvc['web_server_name'] + '/' + propertiesSrvc['services_alias'] + '/anomalies/anomalies/' + anomalyId,
    'headers': {
      'Accept': 'application/x-vm-json'
    },
    'scope': this.$scope_,
    'success': function (response) {
      if (goog.isDefAndNotNull(response.data) && response.data.status == 1) {
        // promise
        deferred.resolve(response.data.data[0]);
      }
    }
  });
  return deferred.promise;
};

/**
 * Zoom l'anomalie selectionné
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.zoomOnExtent = function (geomFeature) {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.zoomOnExtent");

  var olMap = oVmap.getMap().getOLMap();
  var oOptions = {
    size: olMap.getSize(),
    maxZoom: this.properties['anomalies_max_zoom']
  };
  olMap.getView().fit(geomFeature.extent_, oOptions);
}

/**
 * If haveUpdateRight
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.haveUpdateRight = function (anomaly) {
  var haveUpdateRight = false;
  if (!goog.isDefAndNotNull(anomaly)) {
    anomaly = this.anomalySelected;
  }

  if (goog.isDefAndNotNull(anomaly) && goog.isDefAndNotNull(anomaly.user_id))
    haveUpdateRight =  this.bHaveAdminPrivilege || (this.bHaveUserPrivilege && anomaly.user_id == this.currentUserId);
  
  return haveUpdateRight;
}

/**
 * If haveActiveStatut
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.haveActiveStatut = function (anomaly) {
  var haveActiveStatut = false;
  var aActivedStatus = [1,2,3];
  if (!goog.isDefAndNotNull(anomaly)) {
    anomaly = this.anomalySelected;
  }

  if (goog.isDefAndNotNull(anomaly) && goog.isDefAndNotNull(anomaly.status_id)) {
    if (goog.isDefAndNotNull(anomaly.status_id.selectedOption)) {
      haveActiveStatut = aActivedStatus.indexOf(anomaly.status_id.selectedOption.value) !== -1;
    } else {
      haveActiveStatut = aActivedStatus.indexOf(anomaly.status_id) !== -1;
    }
  }

  return haveActiveStatut;
}

/**
 * 
 */
nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.initUiGrid = function () {
  oVmap.log("nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController.prototype.initUiGrid");

  var this_ = this;

  this.$scope_["gridOptions"] = {
    enableRowHeaderSelection: false,
    enableRowSelection: true,
    enableSelectAll: false,
    enableColumnMenus: false,
    enableColumnResizing: true,
    appHeader: false,
    appHeaderTitleBar: false,
    appHeaderSearchForm: false,
    appHeaderOptionBar: false,
    appLoadGridData: true,
    appResizeGrid: true,
    appShowPagination: false,
    enablePagination: false,
    useExternalPagination: false,
    enablePaginationControls: false,
    // paginationPageSize: 5,
    // paginationPageSizes: [5, 10, 20, 50, 100],
    appFooter: false,
    enableColumnMoving: false,
    enableCellEdit: false,
    rowHeight: 35
  };

  // Définition des colonnes de la liste.
  this.$scope_['gridOptions']['columnDefs'] = [
    {
      name: '',
      field: 'actions',
      cellTemplate: '<app-ui-grid-action-anomalies></app-ui-grid-action-anomalies>',
      width: 72,
      enableSorting: false,
      enableColumnResizing: true
    },
    {
      name: 'N°',
      displayName: 'N°',
      field: 'anomalies_id',
      width: 35,
      enableSorting: true,
      type: 'number',
      enableColumnResizing: true,
      headerCellClass: 'vmap_anomalies_anomalies_id'
    },
    {
      name: 'Statut',
      displayName: 'Statut',
      field: 'status_id',
      width: 35,
      enableSorting: true,
      type: 'string',
      enableColumnResizing: true,
      headerCellClass: 'vmap_anomalies_name',
      cellClass: 'text-c',
      headerCellClass: 'text-c',
      cellTemplate: '<div data-app-anomalies-status-column="{{row.entity[col.field]}}"></div>'
    },
    {
      name: 'Titre',
      displayName: 'Titre',
      field: 'theme_short_name',
      width: 124,
      enableSorting: true,
      type: 'string',
      enableColumnResizing: true,
      headerCellClass: 'vmap_anomalies_title',
      cellTemplate: '<div class="multiline-text-in-grid">{{row.entity["theme_short_name"]}} / {{row.entity["title"]}}</div>'
    }
  ];

  this.$scope_.executeActionButtonEvent = function (oRow, sEvent) {
    var sFormDefinitionName = sEvent === 'edit' ? 'update' : sEvent === 'show' ? 'display' : 'display';
    this_.openFormAnomaly(oRow.entity, sFormDefinitionName);
  }

  this.$scope_.zoomOnExtent = function (oRow) {
    this_.zoomOnExtent(oRow.entity.geomFeature);
  }

  this.$scope_.isDisplayAnomaliesOnMap = function () {
    return this_.isDisplayAnomaliesOnMap;
  }
  

  this.$scope_['gridOptions']['data'] = [];
}

// Définit la directive et le controller
oVmap.module.directive('appAnomalies', nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesDirective);
oVmap.module.controller('AppAnomaliesController', nsVmap.nsToolsManager.nsModules.Anomalies.prototype.anomaliesController);

/**
     * appAnomaliesStatusColumn directive.
     * Mise en forme de la colonne "status_id"
     * @param {service} $translate Translate service.
     * @ngInject
     */
vitisApp.appAnomaliesStatusColumnDrtv = function ($translate) {
  return {
    link: function (scope, element, attrs) {
      var popover = null;
      // 1er affichage ou tri de la liste : maj de la mise en forme.
      var clearObserver = attrs.$observe('appAnomaliesStatusColumn', function (value) {
        // Traduction du libellé du statut de la demande.
        if (typeof (scope.$root.anomalies_status_label) == 'undefined')
          scope.$root.anomalies_status_label = [];
        if (typeof (scope.$root.anomalies_status_label[scope.row.entity.status_id]) == 'undefined') {
          $translate([scope.row.entity.status_name]).then(function (translations) {
            scope.$root.anomalies_status_label[scope.row.entity.status_id] = translations[scope.row.entity.status_name];
          });
        }
        // Si le champ est vide : supprime l'icône.
        if (scope.row.entity[scope.col.field] == null || scope.row.entity[scope.col.field] == '')
          element[0].className = '';
        else {
          // Classes css (ui-grid + spécifiques).
          element[0].className = 'ui-grid-cell-contents anomalies-status-circle';
          element[0].style = 'background-color: ' + scope.row.entity.status_color;
          if (goog.isDefAndNotNull(popover)) {
            var bsPopover = $(element).data('bs.popover');
            bsPopover.options.content = scope.$root.anomalies_status_label[scope.row.entity.status_id];
          } else {
            // Création du 'tooltip'.
            popover = $(element).popover({
              'trigger': 'hover',
              'container': 'body',
              'title': function () {
                return '#' + scope.row.entity.status_id;
              },
              // Placement du tooltip à gauche ou à droite suivant la position horizontale de l'élément.
              'placement': function (oPopoverNode, oElementNode) {
                return scope.$root.workspaceTooltipPlacement(oElementNode);
              },
              'content': function () {
                return scope.$root.anomalies_status_label[scope.row.entity.status_id];
              }
            });
          }
        }
      });
      // Attends la suppression du scope.
      scope.$on("$destroy", function () {
        // Supprime le tooltip.
        $(element)["popover"]("destroy");
        // Supprime l'observateur.
        clearObserver();
      });
    }
  };
};
oVmap.module.directive("appAnomaliesStatusColumn", vitisApp.appAnomaliesStatusColumnDrtv);