/* global oVmap, goog, nsVmap */

/**
 * @author: Armand Bahi
 * @Description: Fichier contenant la classe nsVmap.nsMapManager.MapLegend
 * cette classe permet d'afficher la légende
 */

goog.provide('nsVmap.nsMapManager.MapLegend');

goog.require('oVmap');

/**
 * @classdesc
 * Class {@link nsVmap.nsMapManager.MapLegend}: Map legend tool
 * @constructor
 * @export
 */
nsVmap.nsMapManager.MapLegend = function () {
    oVmap.log("nsVmap.nsMapManager.MapLegend");
};

/**
 * Loads the legend
 * @export
 */
nsVmap.nsMapManager.MapLegend.prototype.loadLegend = function (dataPrintBox, compareDataPrintBox) {
    angular.element('#maplegendcontainer').scope()['ctrl']['loadLegend'](dataPrintBox, compareDataPrintBox);
}

/**
 * If legend is loaded (only use for print)
 * @export
 */
nsVmap.nsMapManager.MapLegend.prototype.isLoadedLegend = function () {
    return angular.element('#maplegendcontainer').scope()['ctrl'].isLoadedLegend;
}

/**
 * Allow to hide the unaccessible legend images
 * @param {object} element
 * @export
 */
nsVmap.nsMapManager.MapLegend.prototype.hideBrokenLegend = function (element) {
    oVmap.log("nsVmap.nsMapManager.MapLegend.prototype.hideBrokenLegend");

    $(element).closest('.layer-legend').hide();
};

/************************************************
 ---------- DIRECTIVES AND CONTROLLERS -----------
 *************************************************/

/**
 * map legend directive
 * @return {angular.Directive} The directive specs.
 * @constructor
 * @ngInject
 * @export
 */
nsVmap.nsMapManager.MapLegend.prototype.maplegendDirective = function () {
    oVmap.log("nsVmap.nsMapManager.MapLegend.prototype.maplegendDirective");
    return {
        restrict: 'A',
        scope: {
            'map': '=appMap',
            'lang': '=appLang'
        },
        controller: 'AppMaplegendController',
        controllerAs: 'ctrl',
        bindToController: true,
        templateUrl: oVmap['properties']['vmap_folder'] + '/' + 'template/layers/maplegend.html'
    };
};

/**
 * map legend image node directive (Allow th onmouseover)
 * @return {angular.Directive} The directive specs.
 * @constructor
 * @ngInject
 * @export
 */
nsVmap.nsMapManager.MapLegend.prototype.maplegendImageDirective = function () {
    oVmap.log("nsVmap.nsMapManager.MapLegend.prototype.maplegendImageDirective");
    return {
        link: function (scope, element) {
            var timeout = 500;
            var i = 0;
            $(element).mouseenter(function () {
                i++;
                var ii = angular.copy(i);

                // Si la souris est dedans depuis timeout ms
                setTimeout(function () {
                    if (ii === i) {
                        var myPopover;
                        $('[app-maplegend]').popover({
                            'html': true,
                            'content': '<img src="' + scope['node']['legendURL'] + '" class="img-responsive">',
                            'placement': function (popover) {
                                myPopover = popover;
                                return 'right';
                            }
                        });
                        $('[app-maplegend]').popover('show');

                        // Place la popover
                        setTimeout(function () {
                            var popoverTop = $(myPopover).offset()['top'];
                            var popoverHeight = $(myPopover).height();
                            var popoverBottom = popoverTop + popoverHeight;
                            var bodyHeight = $('body').height();
                            var footerHeight = $('#footer_line').height();
                            var avaliableHeight = bodyHeight - footerHeight;

                            if (popoverBottom > avaliableHeight) {
                                $(myPopover).css('top', popoverTop - (popoverBottom - avaliableHeight) + 'px');
                            }
                        });
                    }
                }, timeout);
            }).mouseleave(function () {
                i++;
                $('[app-maplegend]').popover('destroy');
            });
        }
    };
};

/**
 * map legend controller
 * @param {object} $scope
 * @ngInject
 * @constructor
 * @export
 */
nsVmap.nsMapManager.MapLegend.prototype.maplegendController = function ($scope) {
    oVmap.log("nsVmap.nsMapManager.MapLegend.prototype.maplegendController");

    var this_ = this;

    this.$scope_ = $scope;

    /**
     * @private
     */
    this['oUrls'] = {};

    /**
     * @private
     */
    this['oCompareUrls'] = {};

    /**
     * Nombre de légende chargé
     */
    this['iNbLegendsLoaded'] = [];

    /**
     * Nombre de légende à charger
     */
    this['iNbLegends'] = 0;

    this['currentIdGetLegend'] = new Date().getTime();

    /**
     * The active map
     */
    $scope['activeMap'] = 1;

    /**
     * legend is load (only use for print)
     */
    this.isLoadedLegend = false;

    /**
     * Contains the events stocked by listenLayers()
     */
    this.aListenedEvents_ = [];

    /**
     * Current map scale
     */
    this['scale'];

    /**
     * @type {ol.Map}
     * @private
     */
    this.map_ = oVmap.getMap().getOLMap();

    var i = 0;
    this.map_.on('moveend', function () {
        // Si l'outil d'impression n'est pas activé
        if (!$('#print-select-btn').hasClass('active')) {
            i++;
            var ii = angular.copy(i);
            setTimeout(function () {
                if (i === ii) {
                    this_['scale'] = oVmap.getMap().getScale({
                        'pretty': false
                    });
    
                    var aLayers = this_.map_.getLayers().getArray();
                    var aDynamicLayers = []
                    for (var y = 0; y < aLayers.length; y++) {
                        if (aLayers[y]['values']['is_dynamic_legend']) {
                            aDynamicLayers.push(aLayers[y]);
                        }
                    }

                    // Recharge les couches dynamique de la map comparé
                    if (this_.$scope_["$root"]["compare_enabled"]) {
                        var aLayers = oVmap.getMapCompare().getOLMap().getLayers().getArray();
                        var aDynamicCompareLayers = []
                        for (var y = 0; y < aLayers.length; y++) {
                            if (aLayers[y]['values']['is_dynamic_legend']) {
                                aDynamicCompareLayers.push(aLayers[y]);
                            }
                        }
                    }

                    this_.refreshLegend(aDynamicLayers, aDynamicCompareLayers);
                }
            }, 500);
        }
    }, this);

    if (goog.isDefAndNotNull(oVmap['properties']['vmap'])) {
        /**
         * true if the tool have to be collapsed
         */
        this['legend_collapsed'] = oVmap['properties']['vmap']['legend_collapsed'];
    }

    // évènement lorsque les couches ont changés (avec timeout en cas de changement rapide de plusieurs couches)
    var bCounter = 0;
    oVmap['scope'].$on('layersChanged', function () {
        bCounter++;
        var tempCounter = angular.copy(bCounter);
        setTimeout(function () {
            if (bCounter === tempCounter) {
                this_.loadLegend();
            }
        }, 200);
    });
    setTimeout(function () {
        oVmap['scope'].$broadcast('layersChanged');
    });
};


/**
 * Resize the layertools
 */
nsVmap.nsMapManager.MapLegend.prototype.maplegendController.prototype.resize = function () {
    oVmap.log('nsVmap.nsMapManager.MapLegend.maplegendController.resize');

    setTimeout(function () {
        oVmap.resizeLayerTools();
    });
}

/**
 * Loads the legend
 * @export
 */
nsVmap.nsMapManager.MapLegend.prototype.maplegendController.prototype.loadLegend = function (dataPrintBox, compareDataPrintBox) {
    oVmap.log('nsVmap.nsMapManager.MapLegend.maplegendController.loadLegend');
    if (goog.isDefAndNotNull(dataPrintBox)) {
        this['iNbLegends'] = 0;
        this.isLoadedLegend = false;
    }

    // si dataPrintBox vaut false on ne recharge pas la légende de la carte principale
    if (dataPrintBox !== false) {
        this['oUrls'] = this.getLegendUrls(oVmap.getMap().getOLMap(), null, dataPrintBox);
        this.downloadLegendImgs(this['oUrls'], dataPrintBox);
    }

    // si compareDataPrintBox vaut false on ne recharge pas la légende de la carte de comparaison
    if (compareDataPrintBox !== false) {
        if(this.$scope_["$root"]["compare_enabled"]){
            this['oCompareUrls'] = this.getLegendUrls(oVmap.getMapCompare().getOLMap(), null, compareDataPrintBox);
            this.downloadLegendImgs(this['oCompareUrls'], dataPrintBox, true);
        } else {
            this['oCompareUrls'] = {};
        }
    }
};

/**
 * Refresh the legend
 * @param {array|undefined} aLayers
 * @param {array|undefined} aCompareLayers
 * @export
 */
nsVmap.nsMapManager.MapLegend.prototype.maplegendController.prototype.refreshLegend = function (aLayers, aCompareLayers) {
    oVmap.log('nsVmap.nsMapManager.MapLegend.maplegendController.refreshLegend');

    var oUrls = this.getLegendUrls(this.map_, aLayers);
    this.downloadLegendImgs(oUrls);
    for (var key in oUrls) {
        this['oUrls'][key] = oUrls[key];
    }

    if (this.$scope_["$root"]["compare_enabled"]) {
        oCompareUrls = this.getLegendUrls(oVmap.getMapCompare().getOLMap(), aCompareLayers);
        this.downloadLegendImgs(oCompareUrls);
        if (goog.isDefAndNotNull(oCompareUrls)) {
            for (var key in oCompareUrls) {
                this['oCompareUrls'][key] = oCompareUrls[key];
            }
        }
    }
};

/**
 * Get the legend URLs
 * @param {object} olMap
 * @param {array|undefined} aLayers
 * @param {array|undefined} dataPrintBox Si ce paramètre est passé, ça signifie que la légende est calculé pour l'impression et donc est dynamique
 * @returns {object}
 */
nsVmap.nsMapManager.MapLegend.prototype.maplegendController.prototype.getLegendUrls = function (olMap, aLayers, dataPrintBox) {
    oVmap.log('nsVmap.nsMapManager.MapLegend.maplegendController.getLegendUrls');

    if (!goog.isDefAndNotNull(aLayers)) {
        var aLayers = olMap.getLayers().getArray();
    }
    var oUrls = {};
    var version = '1.1.0';

    // Récupération des URL WMS
    for (var i = 0; i < aLayers.length; i++) {
        if (aLayers[i].get('legend') === true) {
            if (aLayers[i].get('visible') === true) {
                // Si la légende est dynamique on ajoute le paramètre SCALE
                scaleParam = '';
                if (goog.isDefAndNotNull(dataPrintBox) || aLayers[i]['values']['is_dynamic_legend']) {
                    scaleParam = '&SCALE=' + this['scale'];
                }

                if (aLayers[i].get('type') === 'tilewms') {

                    var sLayerName = aLayers[i].get('name');
                    oUrls[sLayerName] = [];

                    for (var ii = 0; ii < aLayers[i].getSource().getUrls().length; ii++) {

                        var sLayers = aLayers[i].getSource().getParams()['LAYERS'];
                        var aLayerParam = [];
                        if (goog.isDefAndNotNull(sLayers)) {
                            aLayerParam = aLayers[i].getSource().getParams()['LAYERS'].split(',');
                        }

                        // Permet de ne pas visualiser les couches non visibles
                        var aActiveSublayers = aLayers[i].get('activeSublayers');
                        if (goog.isDefAndNotNull(aActiveSublayers)) {
                            aLayerParam = aActiveSublayers;
                        }

                        var sMapParams = '';
                        if ((goog.isDefAndNotNull(dataPrintBox) && dataPrintBox !== '') ||
                            (aLayers[i]['values']['is_dynamic_legend'] && oVmap['properties']['vmap']['is_dynamic_legend_if_visible_data'])) {
                            var tileSize = [oVmap['properties']['vmap']['wmsTilesWidth'], oVmap['properties']['vmap']['wmsTilesHeight']];
                            sMapParams = this.getLegendWMSParams(olMap, version, tileSize, dataPrintBox);
                        }

                        for (var iii = 0; iii < aLayerParam.length; iii++) {
                            var separator = aLayers[i].getSource().getUrls()[ii].indexOf('?') !== -1 ? '&' : '?';

                            oUrls[sLayerName].push({
                                'layerId': i,
                                'collapsed': false,
                                'url': aLayers[i].getSource().getUrls()[ii],
                                'layer': aLayerParam[iii],
                                'olLayer': aLayers[i],
                                'layerName': aLayers[i].get('name'),
                                'service_login': goog.isDefAndNotNull(aLayers[i].get('service_login')) ? aLayers[i].get('service_login') : null,
                                'service_password': goog.isDefAndNotNull(aLayers[i].get('service_password')) ? aLayers[i].get('service_password') : null,
                                'legendURL': aLayers[i].getSource().getUrls()[ii] + separator + 'request=GetLegendGraphic&LAYER=' + aLayerParam[iii] + '&FORMAT=image/png&VERSION=' + version + '&service=WMS' + scaleParam + sMapParams
                            });
                        }
                    }
                } else if (aLayers[i].get('type') === 'imagewms') {

                    var sLayerName = aLayers[i].get('name');

                    var sLayers = aLayers[i].getSource().getParams()['LAYERS'];
                    var aLayerParam = [];
                    if (goog.isDefAndNotNull(sLayers)) {
                        aLayerParam = aLayers[i].getSource().getParams()['LAYERS'].split(',');
                    }

                    // Permet de ne pas visualiser les couches non visibles
                    var aActiveSublayers = aLayers[i].get('activeSublayers');
                    if (goog.isDefAndNotNull(aActiveSublayers)) {
                        aLayerParam = aActiveSublayers;
                    }

                    oUrls[sLayerName] = [];

                    var sMapParams = '';
                    if ((goog.isDefAndNotNull(dataPrintBox) && dataPrintBox !== '') ||
                        (aLayers[i]['values']['is_dynamic_legend'] && oVmap['properties']['vmap']['is_dynamic_legend_if_visible_data'])) {
                        var size = olMap.getSize();
                        sMapParams = this.getLegendWMSParams(olMap, version, size, dataPrintBox);
                    }

                    for (var iii = 0; iii < aLayerParam.length; iii++) {

                        var separator = aLayers[i].getSource().getUrl().indexOf('?') !== -1 ? '&' : '?';

                        oUrls[sLayerName].push({
                            'layerId': i,
                            'collapsed': false,
                            'url': aLayers[i].getSource().getUrl(),
                            'layer': aLayerParam[iii],
                            'olLayer': aLayers[i],
                            'layerName': aLayers[i].get('name'),
                            'service_login': goog.isDefAndNotNull(aLayers[i].get('service_login')) ? aLayers[i].get('service_login') : null,
                            'service_password': goog.isDefAndNotNull(aLayers[i].get('service_password')) ? aLayers[i].get('service_password') : null,
                            'legendURL': aLayers[i].getSource().getUrl() + separator + 'request=GetLegendGraphic&LAYER=' + aLayerParam[iii] + '&FORMAT=image/png&VERSION=' + version + '&service=WMS' + scaleParam + sMapParams
                        });
                    }
                }
            }
        }
    }

    return oUrls;
};

/**
 * Download the blob including the login/pass headers on the request
 * @param {object} oUrls
 * @param {object} dataPrintBox
 * @param {boolean} bIsCompare // Don't reset if function execute to map compare
 * @export
 */
nsVmap.nsMapManager.MapLegend.prototype.maplegendController.prototype.downloadLegendImgs = function (oUrls, dataPrintBox, bIsCompare) {
    oVmap.log('nsVmap.nsMapManager.MapLegend.maplegendController.downloadLegendImgs');
    if (goog.isDefAndNotNull(dataPrintBox) && (!goog.isDefAndNotNull(bIsCompare) || bIsCompare === false)) {
        this['currentIdGetLegend'] = new Date().getTime();
        this['iNbLegendsLoaded'][this['currentIdGetLegend']] = 0;
    }

    // Si on est en demande d'impression et qu'il n'y a pas d'url, on considère que la légende est chargé.
    if (goog.isDefAndNotNull(dataPrintBox) && Object.keys(oUrls).length === 0) {
        this.isLoadedLegend = true;
    }

    for (var key in oUrls) {
        for (var i = 0; i < oUrls[key].length; i++) {
            oUrls[key][i]['legendBlobURL'] = this.downloadLegendImg(oUrls[key][i], dataPrintBox);
        }
    }
};

/**
 * Télécharge l'image de la légende et la remplie au format blob sous l'attribut legendBlobURL
 * @param {object} oLegend
 * @export
 */
nsVmap.nsMapManager.MapLegend.prototype.maplegendController.prototype.downloadLegendImg = function (oLegend, dataPrintBox) {
    oVmap.log('nsVmap.nsMapManager.MapLegend.maplegendController.downloadLegendImg');

    if (goog.isDefAndNotNull(dataPrintBox)) {
        var bGuardState = false;
        var currentIdGetLegend = this['currentIdGetLegend'];
        this['iNbLegends']++;
    }

    var this_ = this;
    var xhr = new XMLHttpRequest();
    xhr.responseType = 'blob';
    xhr.onreadystatechange = function () {
        this_.$scope_.$applyAsync(function () {
            // on ne prend pas l'image si la taille est 75 car ça correspond à une image vide.
            if ((xhr.readyState === XMLHttpRequest.DONE && xhr.status === 200)
                && xhr.response.size > 100
                && (xhr.response.type === "image/png" || xhr.response.type === "image/jpg" || xhr.response.type === "image/jpeg")) {
                oLegend['legendBlobURL'] = URL.createObjectURL(xhr.response);
            } else {
                oLegend['legendBlobURL'] = false;
            }

            // On a besoin de vérifier si toute la légende est bien chargé seulement si la demande vient de l'outil d'impression (présence de dataPrintBox)
            // Le readyState à DONE est déclenché plusieurs fois pour la même url, donc pour le compteur on met une sécurité.
            if (goog.isDefAndNotNull(dataPrintBox) && bGuardState === false && (xhr.readyState === XMLHttpRequest.DONE)) {
                bGuardState = true;
                // Si l'image chargé fait partie de la dernière demande de légende.
                if (currentIdGetLegend === this_['currentIdGetLegend']) {
                    this_['iNbLegendsLoaded'][currentIdGetLegend]++;
                } else { // Sinon, on nettoye le compteur (les anciennes demande de légende)
                    delete this_['iNbLegendsLoaded'][currentIdGetLegend];
                }

                if (this_['iNbLegendsLoaded'][this_['currentIdGetLegend']] === this_['iNbLegends']) {
                    this_.isLoadedLegend = true;
                    oVmap['scope'].$broadcast('printMap');
                }
            }
        });
    };
    xhr.open('GET', oLegend['legendURL'], true);
    if (goog.isDefAndNotNull(oLegend['service_login']) && goog.isDefAndNotNull(oLegend['service_password'])) {
        xhr.setRequestHeader("Authorization", "Basic " + btoa(oLegend['service_login'] + ":" + oLegend['service_password']));
    }
    // Par défaut la légende est non visible
    this_.$scope_.$applyAsync(function () {
        oLegend['legendBlobURL'] = false;
    });
    xhr.send();
};

/**
 * Retourne true si au moins une des légendes a été chargée
 * @param {array} aLegends
 * @returns {Boolean}
 * @export
 */
nsVmap.nsMapManager.MapLegend.prototype.maplegendController.prototype.areLegendsLoaded = function (aLegends) {

    var bAreLegendsLoaded = false;
    for (var i = 0; i < aLegends.length; i++) {
        if (aLegends[i]['legendBlobURL'] !== false) {
            bAreLegendsLoaded = true;
        }
    }
    return bAreLegendsLoaded;
};

/**
 * Retourne les paramètres (CRS ou SRS, WIDTH, HEIGHT, BBOX) par rapport à la carte en cours pour l'url de la légende
 * @param {object} olMap
 * @param {string} version
 * @returns {string}
 * @export
 */
nsVmap.nsMapManager.MapLegend.prototype.maplegendController.prototype.getLegendWMSParams = function (olMap, version, size, dataPrintBox) {
    var v13 = ol.string.compareVersions(version, '1.3') >= 0;

    var extent = olMap.getView().calculateExtent(olMap.getSize());

    if (goog.isDefAndNotNull(dataPrintBox) && goog.isDefAndNotNull(dataPrintBox['extent']) && goog.isDefAndNotNull(dataPrintBox['size'])) {
        extent = dataPrintBox['extent'];
        size = dataPrintBox['size'];
    }

    var projection = olMap.getView().getProjection();

    var params = [];
    params[v13 ? 'CRS' : 'SRS'] = projection.getCode();
    
    params['WIDTH'] = size[0];
    params['HEIGHT'] = size[1];
    
    var axisOrientation = projection.getAxisOrientation();
    var bbox;
    if (v13 && axisOrientation.substr(0, 2) == 'ne') {
        bbox = [extent[1], extent[0], extent[3], extent[2]];
    } else {
        bbox = extent;
    }
    params['BBOX'] = bbox.join(',');

    var sMapParams = '';
    if ((goog.isDefAndNotNull(params['SRS']) || goog.isDefAndNotNull(params['CRS'])) &&
        goog.isDefAndNotNull(params['WIDTH']) && goog.isDefAndNotNull(params['HEIGHT']) &&
        goog.isDefAndNotNull(params['BBOX'])) {
        
        for (var paramKey in params) {
            if (params.hasOwnProperty(paramKey)) {
                var param = params[paramKey];
                sMapParams += '&' + paramKey + '=' + param;
            }
        }
    }
    return sMapParams;
};


// Définit la directive et le controller
oVmap.module.directive('appMaplegend', nsVmap.nsMapManager.MapLegend.prototype.maplegendDirective);
oVmap.module.directive('legendImageNode', nsVmap.nsMapManager.MapLegend.prototype.maplegendImageDirective);
oVmap.module.controller('AppMaplegendController', nsVmap.nsMapManager.MapLegend.prototype.maplegendController);
