/* global goog, nsVmap, oVmap, ol, angular, scope, bootbox, vitisApp, destructor_form, oUrlParams */

/**
 * @author: Armand Bahi
 * @Description: Fichier contenant la classe nsVmap.nsToolsManager.Select
 * cette classe permet l'interrogation de couches
 */
goog.provide('nsVmap.nsToolsManager.Select');

goog.require('oVmap');

goog.require('nsVmap.nsToolsManager.BasicSelect');
goog.require('nsVmap.nsToolsManager.AdvancedSelect');
goog.require('ol.interaction.Snap');
goog.require('ol.interaction.Drag');


/**
 * @classdesc
 * Class {@link nsVmap.nsToolsManager.Select}: allow selection
 *
 * @constructor
 * @export
 */
nsVmap.nsToolsManager.Select = function () {

    this.oAdvancedSelect_ = new nsVmap.nsToolsManager.AdvancedSelect();
    this.oBasicSelect_ = new nsVmap.nsToolsManager.BasicSelect();
};

/**
 * Query the given BO by a EWKT geometry and run the given callback with the data param
 * @param {object} opt_options
 * @param {boolean} opt_options.useTableFormat
 * @param {string} opt_options.EWKTGeometry
 * @param {string} opt_options.bo_id
 * @param {object} opt_options.canceler
 * @param {function} opt_options.callback
 */
nsVmap.nsToolsManager.Select.prototype.queryByEWKTGeom = function (opt_options) {
    var scope = angular.element($('#vmap-select-tool')).scope();
    return scope['ctrl'].queryByEWKTGeom(opt_options);
};

/**
 * Get the real scale buffer in metters
 * @param {number} buffer
 * @param {string} geomType
 * @returns {Number}
 */
nsVmap.nsToolsManager.Select.prototype.getRealScaleBuffer = function (buffer, geomType) {
    var scope = angular.element($('#vmap-select-tool')).scope();
    return scope['ctrl'].getRealScaleBuffer(buffer, geomType);
};

/**
 * Creature and return a new feature based on the geojson geometry property
 * @param {object} data
 * @returns {ol.Feature}
 */
nsVmap.nsToolsManager.Select.prototype.extractFeatureFromData = function (data) {
    var scope = angular.element($('#vmap-select-tool')).scope();
    return scope['ctrl'].extractFeatureFromData(data);
};

/**
 * Display the object card
 * @param {object} selection
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.displayObjectCard = function (selection) {
    var scope = angular.element($('#vmap-select-tool')).scope();
    return scope['ctrl'].displayObjectCard(selection);
};

/**
 * Display the edit form
 * @param {object} selection
 * @param {function|undefined} onsubmit
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.displayEditFrom = function (selection, onsubmit) {
    var scope = angular.element($('#vmap-select-tool')).scope();
    return scope['ctrl'].displayEditFrom(selection, onsubmit);
};

/**
 * Remove the duplicate values without taking care of the 'loked' item
 * @param {array} aSelections
 * @returns {array}
 */
nsVmap.nsToolsManager.Select.prototype.removeDuplicateSelections = function (aSelections) {
    var scope = angular.element($('#vmap-select-tool')).scope();
    return scope['ctrl'].removeDuplicateSelections(aSelections);
};

/**
 * Add the query result to this['aSelections'] removing the duplicate selections
 * @param {object} opt_options
 * @param {array<object>} opt_options.data
 * @param {array<object>} opt_options.selection
 * @param {string} opt_options.dataType summary|list|form
 * @param {function} opt_options.callback callback to run at the end of the function
 */
nsVmap.nsToolsManager.Select.prototype.addQueryResult = function (opt_options) {
    var scope = angular.element($('#vmap-select-tool')).scope();
    return scope['ctrl'].addQueryResult(opt_options);
};

/**
 * Download the summary infos in the selection, if the summary infos is already present it will not downloaded
 * @param {array} aSelection
 * @param {function} callback
 */
nsVmap.nsToolsManager.Select.prototype.getSummaryInfos = function (aSelection, callback) {
    var scope = angular.element($('#vmap-select-tool')).scope();
    return scope['ctrl'].getSummaryInfos(aSelection, callback);
};

/**
 * Download the list infos in the selection, if the list infos is already present it will not downloaded
 * @param {array} aSelection
 * @param {function} callback
 */
nsVmap.nsToolsManager.Select.prototype.getListInfos = function (aSelection, callback) {
    var scope = angular.element($('#vmap-select-tool')).scope();
    return scope['ctrl'].getListInfos(aSelection, callback);
};

/**
 * Download the form infos in the selection, if the form infos is already present it will not downloaded
 * @param {array} aSelection
 * @param {function} callback
 */
nsVmap.nsToolsManager.Select.prototype.getFormInfos = function (aSelection, callback) {
    var scope = angular.element($('#vmap-select-tool')).scope();
    return scope['ctrl'].getFormInfos(aSelection, callback);
};

/**
 * Update on database the BO values
 * @param {string} bo_type
 * @param {object} boValues
 * @param {function} successCallback
 * @param {function} errorCallback
 */
nsVmap.nsToolsManager.Select.prototype.updateBOValues = function (bo_type, boValues, successCallback, errorCallback) {
    var scope = angular.element($('#vmap-select-tool')).scope();
    return scope['ctrl'].updateBOValues(bo_type, boValues, successCallback, errorCallback);
};

/**
 * Get the form values to send
 * @param {object} oFormValues
 * @param {object} oFormDefinition
 * @param {string} sFormDefinitionName
 * @param {array|undefined} aParamsToSave
 * @returns {object|null}
 */
nsVmap.nsToolsManager.Select.prototype.getFormData = function (oFormValues, oFormDefinition, sFormDefinitionName, aParamsToSave) {
    var scope = angular.element($('#vmap-select-tool')).scope();
    return scope['ctrl'].getFormData(oFormValues, oFormDefinition, sFormDefinitionName, aParamsToSave);
};

/**
 * Creates a FormData object from oValues
 * @param {object} oValues
 * @returns {FormData}
 */
nsVmap.nsToolsManager.Select.prototype.getFormDataFromValues = function (oValues) {
    var scope = angular.element($('#vmap-select-tool')).scope();
    return scope['ctrl'].getFormDataFromValues(oValues);
};

/**
 * Get the selected BO
 * @returns {scopectrl.sSelectedBo}
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.getSelectedBo = function () {
    var scope = angular.element($('#vmap-basicselect-tool')).scope();
    if( scope === undefined ) {
        return null;
    }
    return scope['ctrl']['sSelectedBo'];
};

/**
 * Suggest if realy want to delete the object and call submitDeleteObject
 * @param {object} selection
 * @param {object} selection.bo_type
 * @param {object} selection.bo_id_value
 * @param {function} callback
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.deleteObject = function (selection, callback) {
    var scope = angular.element($('#vmap-select-tool')).scope();
    return scope['ctrl'].deleteObject(selection, callback);
};

/**
 * Suggest if realy want to delete the objects and call submitDeleteObject
 * @param {array<object>} aSelection
 * @param {function} callback
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.deleteMultipleObjects = function (selection, callback) {
    var scope = angular.element($('#vmap-select-tool')).scope();
    return scope['ctrl'].deleteMultipleObjects(selection, callback);
};

/**
 * Add the selected elements from the table to the card
 * @param {array} aSelection
 * @param {boolean} bReplace true to empty the card before adding items
 * @param {boolean|undefined} bZoomOnFeatures true to zoom on features, default true
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.addToCard = function (sSelection, bReplace, bZoomOnFeatures) {
    var scope = angular.element($('#vmap-select-tool')).scope();
    if( scope === undefined ) {
        return null;
    }
    return scope['ctrl'].addToCard(sSelection, bReplace, bZoomOnFeatures);
};

/**
 * Start the geometry edition
 * @param {object} selection
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.editFeature = function (selection) {
    var scope = angular.element($('#vmap-select-tool')).scope();
    return scope['ctrl'].editFeature(selection);
};

/**
 * Parse the geom title and replace the {{ }}
 * @param {string} sFormTitle
 * @param {object} oScope
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.parseFormTile = function (sFormTitle, oScope) {
    var scope = angular.element($('#vmap-select-tool')).scope();
    return scope['ctrl'].parseFormTile(sFormTitle, oScope);
};

/**
 *
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.clearGetFeatureInfoResult = function () {
    var scope = angular.element($('#vmap-select-tool')).scope();
    return scope['ctrl'].clearGetFeatureInfoResult();
};

/**
 *
 * @param {type} aGetFeatureInfoTree
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.showGetFeatureInfoResultWindow = function (aGetFeatureInfoResult, aCoordinate, bModalOpened) {
    var scope = angular.element($('#vmap-select-tool')).scope();
    return scope['ctrl'].showGetFeatureInfoResultWindow(aGetFeatureInfoResult, aCoordinate, bModalOpened);
};

/**
 * App-specific directive wrapping the select tools.
 * @return {angular.Directive} The directive specs.
 * @constructor
 */
nsVmap.nsToolsManager.Select.prototype.selectDirective = function () {
    oVmap.log("nsVmap.nsToolsManager.Select.prototype.selectDirective");
    return {
        restrict: 'A',
        scope: {
            'map': '=appMap',
            'lang': '=appLang',
            'currentAction': '=appAction'
        },
        controller: 'AppSelectController',
        controllerAs: 'ctrl',
        bindToController: true,
        templateUrl: oVmap['properties']['vmap_folder'] + '/' + 'template/tools/select.html'
    };
};

/**
 * @ngInject
 * @constructor
 * @param {object} $scope
 * @param {object} $timeout
 * @param {object} $q
 * @returns {nsVmap.nsToolsManager.Select.prototype.selectController}
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController = function ($scope, $timeout, $q, $element, $translate, $sce) {
    oVmap.log("nsVmap.nsToolsManager.Select.prototype.selectController");

    var this_ = this;

    /**
     * @private
     */
    this.$scope_ = $scope;

    /**
     * @private
     */
    this.$timeout_ = $timeout;

    /**
     * @private
     */
    this.$q_ = $q;

    /**
     * Dummy canceler (use canceler.resolve() for cancel the ajax request)
     * @private
     */
    this.canceler_ = $q.defer();

    /**
     * @private
     */
    this.$translate_ = $translate;

    /**
     * @private
     */
    this.$sce_ = $sce;

    /**
     * The current properties
     */
    this['properties'] = oVmap['properties'];

    /**
     * The current token
     */
    this['token'] = oVmap['properties']['token'];

    /**
     * Type de géométrie à utiliser
     */
    this['addPartGeomType'] = "";

    /**
     * Définit si l'échélle est dans la tranche allouée par l'objet métier
     */
    this['isMaxScaleOk'] = true;

    /**
     * Définit si l'échélle est dans la tranche allouée par l'objet métier
     */
    this['isMinScaleOk'] = true;

    /**
     * Booléen représentant l'ouverture du SnapMenu
     */
    this['isSnapMenuCollapse'] = false;

    /**
     * Object contenant les diverses options de snapping
     */
    this['snapOptions'] = {
        'mode': 'segment_edge_node',
        'tolerance': 15,
        'limit': 2000,
        'visible': false
    };
    if (goog.isDefAndNotNull(oVmap['properties']['snapping'])) {
        this['snapOptions']['tolerance'] = oVmap['properties']['snapping']['defaut_tolerance'];
        this['snapOptions']['mode'] = oVmap['properties']['snapping']['defaut_snapp_mode'];
        this['snapOptions']['limit'] = oVmap['properties']['snapping']['defaut_limit'];
        this['snapOptions']['visible'] = oVmap['properties']['snapping']['defaut_visibility'];
        this['snapOptions']['avoidSuperpositions'] = {};
    }

    /**
     * Object temporaire permettant l'édition des diverses options de snapping
     */
    this['tmpSnapOptions'] = angular.copy(this['snapOptions']);

    /**
     * Queryable Business Objects
     */
    this['aQueryableBOs'] = [];

    /**
    * Business objects avaliable for avoiding superpositions
    */
    $scope['aAvoidSuperpositionsBOs'] = [];

    /**
     * Tree resulting the various GetFeatureInfo requests
     */
    this['aGetFeatureInfoResult'] = [];

    /**
     * The selected layer index
     */
    this['aGetFeatureInfoSelectedLayer'] = 0;

    /**
     * The selected sublayer index
     */
    this['aGetFeatureInfoSelectedSubLayer'] = 0;

    /**
     * The number of layers witch have results on GetCapabilities
     */
    this['iGetFeatureInfoWithResultLayers'] = 0;

    /**
     * The number of layers witch not have results on GetCapabilities
     */
    this['iGetFeatureInfoWithoutResultLayers'] = 0;

    /**
     * Maximum popup mode objects
     * @type integer
     */
    this.limitPopup_ = oVmap['properties']["selection"]["limit_popup"];

    /**
     * Maximum list mode objects
     * @type integer
     */
    this.limitList_ = oVmap['properties']["selection"]["limit_list"];

    /**
     * Form values
     */
    this['oFormValuesUpdate'];

    /**
     * Is clonnable bo
     * @type boolean
     */
    this['isClonnableBO'] = false;

    /**
     * Geometry is editable
     * @type boolean
     */
    this['isEditableGeometry'] = true;

    /**
     * @type {ol.layer.Vector}
     * @private
     */
    this.oOverlayLayer_ = oVmap.getMap().addVectorLayer({
        style: new ol.style.Style({
            fill: new ol.style.Fill({
                color: 'rgba(255, 255, 255, 0.6)'
            }),
            stroke: new ol.style.Stroke({
                color: '#319FD3',
                width: 2
            }),
            image: new ol.style.Circle({
                radius: 7,
                fill: new ol.style.Fill({
                    color: '#319FD3'
                })
            })
        })
    });

    /**
     * @type {ol.Collection}
     * @private
     */
    this.oOverlayFeatures_ = this.oOverlayLayer_.getSource().getFeaturesCollection();

    /**
     * Overlay contenant les trous des polygones (invisible)
     * @type {ol.layer.Vector}
     * @private
     */
    this.oOverlayHolesLayer_ = oVmap.getMap().addVectorLayer({
        style: new ol.style.Style({
            fill: new ol.style.Fill({
                color: 'rgba(0, 0, 0, 0.2)'
            })
        })
    });

    /**
     * @type {ol.layer.Vector}
     * @private
     */
    this.oSnapOverlayLayer_ = new ol.layer.Vector({
        map: oVmap.getMap().getOLMap(),
        visible: false,
        source: new ol.source.Vector({
            useSpatialIndex: false
        }),
        style: new ol.style.Style({
            fill: new ol.style.Fill({
                color: 'rgba(255, 255, 255, 0.2)'
            }),
            stroke: new ol.style.Stroke({
                color: '#1000ff',
                width: 2
            }),
            image: new ol.style.Circle({
                radius: 3,
                fill: new ol.style.Fill({
                    color: '#1000ff'
                })
            })
        }),
    });

    /**
     * @type {ol.interaction.Snap}
     * @private
     */
    this.snap_ = new ol.interaction.Snap({
        vertex: true,
        edge: (this['snapOptions']['mode'] === 'segment_edge_node' ? true : false),
        pixelTolerance: this['snapOptions']['tolerance'],
        source: this.oSnapOverlayLayer_.getSource()
    });

    /**
     * @type {ol.interaction.Drag}
     * @private
     */
    this.drag_ = new ol.interaction.Drag();

    /**
     * @type {ol.layer.Vector}
     * @private
     */
    this.oGetFeatureInfoOverlay_ = new ol.layer.Vector({
        map: oVmap.getMap().getOLMap(),
        source: new ol.source.Vector({
            useSpatialIndex: false
        }),
        style: new ol.style.Style({
            fill: new ol.style.Fill({
                color: 'rgba(255, 255, 255, 0.2)'
            }),
            stroke: new ol.style.Stroke({
                color: '#1000ff',
                width: 2
            }),
            image: new ol.style.Circle({
                radius: 5,
                fill: new ol.style.Fill({
                    color: '#1000ff'
                })
            })
        }),
    });

    /**
     * @type {ol.interaction.Modify}
     * @private
     */
    this.modify_ = new ol.interaction.Modify({
        features: this.oOverlayFeatures_
    });

    this.modify_.on('modifystart', function (evt) {

        // Récupère la feature ajoutée
        var aFeatures = evt.features.getArray();

        var j = 0;
        for (var i = 0; i < aFeatures.length; i++) {

            // Événement permettant de changer la tooltip de mesure lorsqu'on modifie la geom
            if (aFeatures[i].getGeometry().get('changeTooltipEvent') !== true) {
                aFeatures[i].getGeometry().on('change', function (event) {
                    oVmap.getMap().createMeasureTooltip(event.target);
                });
                aFeatures[i].getGeometry().set('changeTooltipEvent', true);
            }
        }
    });

    this.modify_.on('modifyend', function (evt) {

        // Récupère la feature ajoutée
        var aFeatures = evt.features.getArray();

        var j = 0;
        for (var i = 0; i < aFeatures.length; i++) {
            oVmap.getMap().createMeasureTooltip(aFeatures[i].getGeometry());
            // Évite les superpositions avec les autres couches si besoin
            this_.avoidSuperpositions(aFeatures[i]).then(function(){
                j++;
                if (!j < aFeatures.length) {
                    setTimeout(function () {
                        // Enregistre les changements
                        this_.putFeaturesOnTheElement(this_.oOverlayFeatures_.getArray());
                    });
                }
            });
        }

        // Cas où avoidSuperpositions n'ait pas effectué de callback
        setTimeout(function () {
            if (j === 0) {
                console.error('avoidSuperpositions never sents callback');
                // Enregistre les changements
                this_.putFeaturesOnTheElement(this_.oOverlayFeatures_.getArray());
            }
        }, 3000);
    });

    /**
     * @type {ol.interaction.Select}
     * @private
     */
    this.delete_ = new ol.interaction.Select({
        layers: [this.oOverlayLayer_]
    });

    // Supprime la feature lors de la sélection sur l'interaction this.delete_
    this.delete_.on('select', function () {
        var aSelectedFeatures = this.delete_.getFeatures().getArray();
        for (var i = 0; i < aSelectedFeatures.length; i++) {
            this.oOverlayLayer_.getSource().removeFeature(aSelectedFeatures[i]);
        }
        this.delete_.getFeatures().clear();
        setTimeout(function () {
            this_.putFeaturesOnTheElement(this_.oOverlayFeatures_.getArray());
        });
    }, this);

    /**
     * Outil de suppression de trous
     * @type {ol.interaction.Select}
     * @private
     */
    this.deleteHole_ = new ol.interaction.Select({
        layers: [this.oOverlayHolesLayer_]
    });

    // Supprime du trou de la feature (si il existe) feature lors de la sélection sur l'interaction this.deleteHole_
    this.deleteHole_.on('select', this.deleteHole, this);

    // Anulle l'édition quand on change d'outil
    oVmap['scope'].$on('toggleOutTools', function () {
        this_.clearOverlays();
        this_.canceler_.resolve();
    });

    // Zoom sur les objets passés dans l'url
    if (goog.isString(oUrlParams['bo_id']) && goog.isString(oUrlParams['ids'])) {
        if (oUrlParams['bo_id'].length > 0 && oUrlParams['ids'].length > 0) {
            setTimeout(function () {

                var oBusinessObjects = oVmap.getMapManager().getQueryableBusinessObjects();

                if (goog.isDefAndNotNull(oBusinessObjects[oUrlParams['bo_id']])) {

                    var aSelection = [];
                    var aIds = oUrlParams['ids'].split('|');

                    for (var i = 0; i < aIds.length; i++) {
                        aSelection.push({
                            'bo_type': oUrlParams['bo_id'],
                            'bo_id_value': aIds[i],
                            'bo_id_field': oBusinessObjects[oUrlParams['bo_id']]['bo_id_field'],
                            'bo_title': oBusinessObjects[oUrlParams['bo_id']]['bo_title']
                        });
                    }

                    if (aSelection.length > 0) {
                        this_.addToCard(aSelection, true);
                    }
                }
            });
        }
    }

    // Á chaque fois qui l'utilisateur finit de bouger la carte
    oVmap.getMap().getOLMap().on('moveend', function () {
        // Vérifie si l'échelle d'édition est bonne
        this_.checkEditionScale();
        // Lance la fonction de rechargement
        this_.loadVectorSnappingData();
    });

    // Rempli les bo disponibles
    oVmap['scope'].$on('layersChanged', function () {
        this_.checkEditionScale();
        this_.loadQueryableBos();
        this_.closeGetFeatureInfoResultWindow();
    });
    oVmap['scope'].$on('selectedBoChanged', function () {
        this_.checkEditionScale();
        this_.loadQueryableBos();
    });
    setTimeout(function () {
        this_.loadQueryableBos();
    });

    // Affiche les modales en plein écran pour la version mobile
    if (oVmap['properties']['is_mobile']) {
        $element.find('.modal').on('shown.bs.modal', function () {
            $('.modal-backdrop.fade.in').hide();
            $('.modal.fade.in').find('.modal-dialog').addClass('mobile-full-modal');
        });
    }

    // Permet de savoir si on est en modification (clic soutenu)
    this.isModifying_ = false;
    this.modify_.on('modifyend', function () {
        this_.isModifying_ = false;
        this_.checkEditionScale();
    });
    this.modify_.on('modifystart', function () {
        this_.isModifying_ = true;
    });

    // Fornction permettant de trier la liste des couches d'un calque dans le fenêtre oGetFeatureInfo
    $scope['sortGetFeatureInfoSubLayers'] = function(oSubLayer) {
        if (oSubLayer['index'] === this_['aGetFeatureInfoSelectedSubLayer'] && $scope['collapseSublayersTabs'] === true) {
            return -1;
        }
        return oSubLayer['index'];
    }

    /**
     *
     */
    this['selectTypeInsertGeom'] = 'WKT';
    this['onlySelectWKT'] = false;

    this['addGeomPointX'] = '';
    this['addGeomPointY'] = '';
    this['addGeomWKT'] = '';
    this['addGeomProj'] = '';

    this['projections'] = oVmap['oProjections'];
    $scope['oAuthorizedGeomType'] = oVmap.getMapManager().getAuthorizedGeomType();
};

/**
 * Charge la liste des objets métier
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.loadQueryableBos = function () {
    oVmap.log('nsVmap.nsToolsManager.Select.selectController.loadQueryableBos');
    var this_ = this;

    this_.$scope_.$applyAsync(function () {
        setTimeout(function () {
            this_['aQueryableBOs'] = oVmap.getMapManager().getQueryableBusinessObjectsAsArray(true, true, false, false, false);

            // Snapping
            for (var i = 0; i < this_['aQueryableBOs'].length; i++) {
                this_['aQueryableBOs'][i]['loadCanceller'] = this_.$q_.defer();
                this_['aQueryableBOs'][i]['isMouseOverSnaoMenuOb'] = false;
                this_['aQueryableBOs'][i]['bo_snapping_loaded'] = null;
                if (this_['aQueryableBOs'][i]['bo_id'] === oVmap.getToolsManager().getBasicTools().getSelect().getSelectedBo()) {
                    this_['aQueryableBOs'][i]['bo_snapping_enabled'] = true;
                } else {
                    this_['aQueryableBOs'][i]['bo_snapping_enabled'] = false;
                }
            }

            // Mobile: cache le bouton "Insertion" si aucun bo est insertable
            if (goog.isDefAndNotNull(this_['aQueryableBOs'])) {
                if (oVmap['properties']['is_mobile']) {
                    if (this_['aQueryableBOs'].length > 0) {
                        $('#vmap_menu_mobile_menu_requeteur_button').show();
                    } else {
                        $('#vmap_menu_mobile_menu_requeteur_button').hide();
                    }

                    //Modifie la taille de la liste des objets métiers pour ne pas qu'elle soit trop large
                    angular.element("#select-bo-select")[0].style["width"] = "23vw";
                }
            }
        });
    });
};

/**
 * Load this.aAvoidSuperpositionsBOs
 *
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.loadAvoidSuperpositionBos = function () {
    oVmap.log('nsVmap.nsToolsManager.Select.selectController.loadAvoidSuperpositionBos');

    var this_ = this;

    // Types de géométries pour le calcul des jointures
    var aActiveGeomTypes = ['POLYGON', 'MULTIPOLYGON', 'GEOMETRY', 'GEOMETRYCOLLECTION'];

    // Objets métiers actifs pour la calcul des jointures
    this.$scope_['aAvoidSuperpositionsBOs'] = [];

    if (goog.isDefAndNotNull(this['editableFeatureType'])) {

        // Si la couche en cours d'insertion est dans aActiveGeomTypes
        if (aActiveGeomTypes.indexOf(this['editableFeatureType']) !== -1) {
            for (var i = 0; i < this['aQueryableBOs'].length; i++) {

                // Si la couche cible est dans aActiveGeomTypes
                if (goog.isDefAndNotNull(this['aQueryableBOs'][i]['bo_geom_type'])) {
                    if(aActiveGeomTypes.indexOf(this['aQueryableBOs'][i]['bo_geom_type']) !== -1){

                        this.$scope_['aAvoidSuperpositionsBOs'].push(this['aQueryableBOs'][i]);
                    }
                }
            }
        }
    }
}

/**
 * Vérifie que l'échelle en cours respecte les spécification de l'objet métier
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.checkEditionScale = function () {
    oVmap.log('nsVmap.nsToolsManager.Select.selectController.checkEditionScale');

    if (!goog.isDefAndNotNull(this['aQueryableBOs'])) {
        return;
    }

    var oBo;
    for (var i = 0; i < this['aQueryableBOs'].length; i++) {
        if (this['aQueryableBOs'][i]['bo_id'] === oVmap.getToolsManager().getBasicTools().getSelect().getSelectedBo()) {
            oBo = this['aQueryableBOs'][i];
            break;
        }
    }

    if (!goog.isDefAndNotNull(oBo)) {
        return;
    }

    var maxScale = oBo['bo_max_edition_scale'];
    var minScale = oBo['bo_min_edition_scale'];

    if (this['currentAction'] === 'basicTools-select-editFeature') {
        if (goog.isDefAndNotNull(minScale)) {
            minScale = parseFloat(minScale);
            if (oVmap.getMap().getScale() < minScale && !this.isModifying_) {
                oVmap.getMap().getMapTooltip().displayMessage('Échelle minimale de saisie atteinte, dé-zoomez pour continuer l\'édition');
                this['isMinScaleOk'] = false;
                this.modify_.setActive(false);
            } else {
                this['isMinScaleOk'] = true;
            }
        }
        if (goog.isDefAndNotNull(maxScale)) {
            maxScale = parseFloat(maxScale);
            if (oVmap.getMap().getScale() > maxScale && !this.isModifying_) {
                oVmap.getMap().getMapTooltip().displayMessage('Échelle maximale de saisie atteinte, zoomez pour continuer l\'édition');
                this['isMaxScaleOk'] = false;
                this.modify_.setActive(false);
            } else {
                this['isMaxScaleOk'] = true;
            }
        }
        if (this['isMinScaleOk'] && this['isMaxScaleOk']) {
            if (!this.modify_.getActive()) {
                this.modify_.setActive(true);
            }
            if (this.modify_.getActive()) {
                oVmap.getMap().getMapTooltip().displayMessage('Cliquez sur la géométrie pour la modifier');
            }
        }
    }
};

/**
 * Query the given BO by a EWKT geometry and run the given callback with the data param
 * @param {object} opt_options
 * @param {boolean} opt_options.useTableFormat
 * @param {string} opt_options.EWKTGeometry
 * @param {string} opt_options.sFilter
 * @param {string} opt_options.bo_id
 * @param {object} opt_options.canceler
 * @param {function} opt_options.callback
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.queryByEWKTGeom = function (opt_options) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.queryByEWKTGeom');

    var this_ = this;

    if (!goog.isDefAndNotNull(opt_options)) {
        return;
    }
    if (!goog.isDefAndNotNull(opt_options.useTableFormat)) {
        console.error('opt_options.useTableFormat not defined');
        return;
    }
    if (!goog.isDefAndNotNull(opt_options.EWKTGeometry)) {
        console.error('opt_options.EWKTGeometry not defined');
        return;
    }
    if (!goog.isDefAndNotNull(opt_options.bo_id)) {
        console.error('opt_options.bo_id not defined');
        return;
    }
    if (!goog.isDefAndNotNull(opt_options.canceler)) {
        console.error('opt_options.canceler not defined');
        return;
    }
    if (!goog.isDefAndNotNull(opt_options.callback)) {
        console.error('opt_options.callback not defined');
        return;
    }

    // Si on est en mode tableFormat, alors on ne demande pas la géométrie
    var get_geom = !opt_options.useTableFormat;

    // Type de résultat: list ou summary
    var formType = opt_options.useTableFormat === true ? 'list' : 'summary';

    // Projection en cours
    var srid = this['map'].getView().getProjection().getCode().substring(5);

    // Récupère le type de géométrie
    var geomType = oVmap.getGeomFromEWKT(opt_options.EWKTGeometry).getType();

    // Calcul du buffer
    var buffer = this.getRealScaleBuffer(opt_options.buffer, geomType);

    // Limite de features
    if (opt_options.useTableFormat === true)
        var limit = this.limitList_;
    else
        var limit = this.limitPopup_;

    ajaxRequest({
        'method': 'POST',
        'url': oVmap['properties']['api_url'] + '/vmap/querys/' + opt_options.bo_id + '/' + formType,
        'headers': {
            'X-HTTP-Method-Override': 'GET',
            'Accept': 'application/x-vm-json'
        },
        'data': {
            'filter': opt_options.sFilter,
            'intersect_geom': opt_options.EWKTGeometry,
            'intersect_buffer': buffer,
            'get_geom': get_geom,
            'get_image': get_geom,
            'result_srid': srid,
            'limit': limit,
            'd': Date.now()
        },
        'scope': this.$scope_,
        'abord': opt_options.canceler.promise,
        'success': function (response) {

            // Si le opt_options.canceler à été resolved (on a appuillé sur echap ou on à lancé une autre requête)
            if (opt_options.canceler.promise['$$state'].status === 1)
                return 0;

            // Résultat vide par défaut, il faut passer par addQueryResult même en cas de résultat vide
            var data = [];

            // Vérifie si il y a des données à afficher et les ajoute
            if (goog.isDefAndNotNull(response['data'])) {
                if (goog.isDefAndNotNull(response['data']['data'])) {
                    data = response['data']['data'];
                } else {
                    oVmap.log('Pas de données à afficher');
                }
            } else {
                oVmap.log('Pas de données à afficher');
            }

            if (data.length === this_.limitList_)
                $.notify("Limite d'objets à afficher atteinte", "warn");

            opt_options.callback.call(this, data);
        }
    });
};

/**
 * Get the real scale buffer in metters
 * @param {number} buffer
 * @param {string} geomType
 * @returns {Number}
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.getRealScaleBuffer = function (buffer, geomType) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.getRealScaleBuffer');

    if (!goog.isDefAndNotNull(buffer)) {
        return null;
    }
    if (!goog.isDefAndNotNull(geomType)) {
        return null;
    }

    var iAPIBuffer = null;
    if (geomType === 'Point' || geomType === 'MultiPoint') {
        if (goog.isDefAndNotNull(buffer)) {
            var scale = oVmap.getMap().getScale();
            iAPIBuffer = (buffer * scale) / 1000;
        }
    }
    return iAPIBuffer;
};

/**
 * Creature and return a new feature based on the geojson geometry property
 * @param {object} data
 * @returns {ol.Feature}
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.extractFeatureFromData = function (data) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.extractFeatureFromData');

    // Récupère la géométrie
    var geom = oVmap.getGeomFromEWKT(data['bo_intersect_geom']);
    var feature = new ol.Feature({
        geometry: geom
    });

    return feature;
};

/**
 * Get the BasicSelect scope
 * @returns {angular.scope}
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.getBasicSelectScope = function () {
    return angular.element($('#vmap-basicselect-tool')).scope();
};

/**
 * Get the AdvancedSelect scope
 * @returns {angular.scope}
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.getAdvancedSelectScope = function () {
    return angular.element($('#vmap-advancedselect-tool')).scope();
};

/**
 * Get the Display Form reader scope
 * @returns {angular.scope}
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.getDisplayFormReaderScope = function () {
    return angular.element($('#select_display_form_reader').children()).scope();
};

/**
 * Get the Edit Form reader scope
 * @returns {angular.scope}
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.getEditFormReaderScope = function () {
    return angular.element($('#select_edit_form_reader').children()).scope();
};

/**
 * Display the object card
 * @param {object} selection
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.displayObjectCard = function (selection) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.displayObjectCard');

    var this_ = this;

    if (selection['bo_title'].length === 0)
        return 0;

    this['editSelection'] = selection;

    // Récupere les informations du formuaire
    this_.getFormInfos([selection], function (data) {

        selection = data[0];

        var oFormDefinition = selection['bo_json_form'];

        var oFormValues = {
            "update": goog.object.clone(selection['bo_form']),
            "display": goog.object.clone(selection['bo_form'])
        };

        oVmap.log('oFormValues: ', oFormValues);
        oVmap.log('oFormDefinition: ', oFormDefinition);

        var displayFormReaderScope = this_.getDisplayFormReaderScope();
        displayFormReaderScope.$evalAsync(function () {

            var loadForm = function () {
                // Set le formulaire
                displayFormReaderScope['ctrl']['setDefinitionName']('display');
                displayFormReaderScope['ctrl']['setFormValues'](oFormValues);
                displayFormReaderScope['ctrl']['setFormDefinition'](oFormDefinition);

                displayFormReaderScope.$applyAsync(function () {
                    displayFormReaderScope['ctrl']['loadForm']();
                    $('#select-card-modal').modal('show');
                });
            };

            // Tente de lancer destructor_form
            try {
                if (goog.isDef(destructor_form)) {
                    destructor_form();
                }
            } catch (e) {
                oVmap.log("destructor_form does not exist");
            }

            // Chargement des js associés au bo
            if (goog.isDefAndNotNull(selection['bo_json_form_js'])) {
                var sUrl = selection['bo_json_form_js'];
                oVmap.log("initHtmlForm : javascript assoc. to : " + sUrl);
                loadExternalJs([sUrl], {
                    "callback": function () {
                        loadForm();
                        try {
                            if (goog.isDef(constructor_form)) {
                                constructor_form(displayFormReaderScope, sUrl);
                            }
                        } catch (e) {
                            oVmap.log("constructor_form does not exist");
                        }
                    },
                    "async": true,
                    "scriptInBody": true
                });
            } else {
                loadForm();
            }
        });
    });
};

/**
 * Display the edit form
 * @param {object|array} selection
 * @param {function|undefined} onsubmit
 * @param {boolean|undefined} isMultipleEdit
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.displayEditFrom = function (selection, onsubmit, isMultipleEdit) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.displayEditFrom');

    if (!goog.isDefAndNotNull(isMultipleEdit)) {
        isMultipleEdit = false; // default value
    }

    var this_ = this;

    var typeForm = 'update';

    if (isMultipleEdit) {
        if (selection.length === 0)
            return 0;

        this['isClonnableBO'] = false;
        this['isEditableGeometry'] = false;
        typeForm = 'multi-update';
    } else {
        if (selection['bo_title'].length === 0)
            return 0;

        this['editSelection'] = selection;

        // Init isClonnableBO
        var aInsertableBOs = oVmap.getMapManager().getInsertableBusinessObjects(true, true, false);
        var iIndex = aInsertableBOs.findIndex(function(bo) {
            return (bo['bo_id'] === selection['bo_type'] && bo['bo_enable_clone'] === true);
        });

        this['isClonnableBO'] = (iIndex !== -1) ;
        this['isEditableGeometry'] = true;

        for (var i = 0; i < this['aQueryableBOs'].length; i++) {
            if (this['aQueryableBOs'][i]['bo_id'] === selection['bo_type']) {
                var oBo = this['aQueryableBOs'][i];

                if (!goog.isDefAndNotNull(oBo['bo_geom_column']) || !oBo['bo_geom_column'].length > 0 || oBo['bo_enable_geom_updater'] == false) {
                    this['isEditableGeometry'] = false;
                }
            }
        }

        selection = [selection];
    }

    // Récupere les informations du formuaire
    this_.getSummaryInfos(selection, function (aSelection) {
        this_.getFormInfos(aSelection, function (aSelection) {

            selection = aSelection[0];

            if (!goog.isDefAndNotNull(this_['editSelection']['bo_formtitle'])) {
                this_['editSelection']['bo_formtitle'] = selection['bo_formtitle'];
            }
            if (!goog.isDefAndNotNull(this_['editSelection']['bo_summarytitle'])) {
                this_['editSelection']['bo_summarytitle'] = selection['bo_summarytitle'];
            }
            if (!goog.isDefAndNotNull(this_['editSelection']['add_form_size'])) {
                this_['editSelection']['add_form_size'] = selection['add_form_size'];
            }
            if (!goog.isDefAndNotNull(this_['editSelection']['edit_form_size'])) {
                this_['editSelection']['edit_form_size'] = selection['edit_form_size'];
            }
            if (!goog.isDefAndNotNull(this_['editSelection']['display_form_size'])) {
                this_['editSelection']['display_form_size'] = selection['display_form_size'];
            }

            var oFormDefinition = selection['bo_json_form'];
            var oFormValues = {
                "update": goog.object.clone(selection['bo_form']),
                "multi-update": [],
                "display": goog.object.clone(selection['bo_form'])
            };

            // Si c'est une edition multiple on vérifie que le formulaire existe.
            if (isMultipleEdit && !goog.isDefAndNotNull(oFormDefinition['multi-update'])) {
                return 0;
            }

            this_.oFormValuesUpdate = oFormValues['update'];

            // Bouton submit
            var button = {
                "fields": [{
                        "type": "button",
                        "class": "btn-ungroup btn-group-sm",
                        "nb_cols": 12,
                        "buttons": [{
                                "visibleAllTabs": true,
                                "type": "submit",
                                "label": "FORM_UPDATE",
                                "name": "form_submit",
                                "class": "btn-primary hidden",
                                "id": "basictools-select-form-reader-submit-btn"
                            }
                        ]
                    }
                ]
            };
            if (!angular.equals(oFormDefinition[typeForm]['rows'][oFormDefinition[typeForm]['rows'].length - 1], button))
                oFormDefinition[typeForm]['rows'].push(button);

            // Evènement submit
            oFormDefinition[typeForm]['event'] = function (oFormValuesResulted) {

                $('#select_edit_form_reader').find('button').attr('disabled', true);
                showAjaxLoader();

                var aParamsToSave = [];
                if (!isMultipleEdit) {
                    // Paramètres peut être non fournis dans le formulaire à sauvegarger
                    var aParamsToSave = [selection['bo_id_field']];
                    if (goog.isDefAndNotNull(this_['editableSelection'])) {
                        if (goog.isDefAndNotNull(this_['editableSelection']['geom_column'])) {
                            aParamsToSave.push(this_['editableSelection']['geom_column']);
                        }
                    }
                }

                var boValues = this_.getFormData(oFormValuesResulted, oFormDefinition, typeForm, aParamsToSave);

                selection['bo_form'] = boValues;

                if (isMultipleEdit) {
                    // Si édition multiple on ne garde pas les valeurs non renseigné.
                    for (var key in boValues) {
                        if (boValues[key].length === 0 || boValues[key] === '') {
                            delete boValues[key];
                        }
                    }

                    // Pour toute les valeurs sauf la dernière, on fait une simple mise à jour
                    for(var i=0; i < aSelection.length - 1; i++) {
                        boValues[aSelection[i]['bo_id_field']] = aSelection[i]['bo_id_value'];
                        this_.updateBOValues(selection['bo_type'], boValues);
                    }

                    // On configure boValues pour mettre a jour le dernier enregistrement modifier
                    boValues[aSelection[aSelection.length-1]['bo_id_field']] = aSelection[aSelection.length-1]['bo_id_value'];
                }

                this_.updateBOValues(selection['bo_type'], boValues, function () {
                    $('#select-edit-modal').modal('hide');
                    $('#select_edit_form_reader').find('button').attr('disabled', false);
                    this_.clearOverlays();
                    if (goog.isDefAndNotNull(onsubmit)) {
                        onsubmit.call(this, boValues);
                    }
                }, function () {
                    $('#select-edit-modal').modal('hide');
                    $('#select_edit_form_reader').find('button').attr('disabled', false);
                    this_.clearOverlays();
                    if (goog.isDefAndNotNull(onsubmit)) {
                        onsubmit.call(this, boValues);
                    }
                });

            };


            var editFormReaderScope = this_.getEditFormReaderScope();
            editFormReaderScope.$evalAsync(function () {

                var loadForm = function () {
                    // Set le formulaire
                    editFormReaderScope['ctrl']['setDefinitionName'](typeForm);
                    editFormReaderScope['ctrl']['setFormValues'](oFormValues);
                    editFormReaderScope['ctrl']['setFormDefinition'](oFormDefinition);

                    editFormReaderScope.$applyAsync(function () {
                        editFormReaderScope['ctrl']['loadForm']();
                        $('#select-edit-modal').modal('show');
                    });
                };

                // Tente de lancer destructor_form
                try {
                    if (goog.isDef(destructor_form)) {
                        destructor_form();
                    }
                } catch (e) {
                    oVmap.log("destructor_form does not exist");
                }

                // Chargement des js associés au bo
                if (goog.isDefAndNotNull(selection['bo_json_form_js'])) {
                    var sUrl = selection['bo_json_form_js'];
                    oVmap.log("initHtmlForm : javascript assoc. to : " + sUrl);
                    loadExternalJs([sUrl], {
                        "callback": function () {
                            loadForm();
                            try {
                                if (goog.isDef(constructor_form)) {
                                    constructor_form(editFormReaderScope, sUrl);
                                }
                            } catch (e) {
                                oVmap.log("constructor_form does not exist");
                            }
                        },
                        "async": true,
                        "scriptInBody": true
                    });
                } else {
                    loadForm();
                }
            });
        });
    });
};

/**
 * Download the summary infos in the selection, if the summary infos is already present it will not downloaded
 * @param {array} aSelection
 * @param {function} callback
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.getSummaryInfos = function (aSelection, callback) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.getSummaryInfos');
    this.complementDataOnSelection({
        aSelection: aSelection,
        dataType: 'summary',
        callback: callback
    });
};

/**
 * Download the list infos in the selection, if the list infos is already present it will not downloaded
 * @param {array} aSelection
 * @param {function} callback
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.getListInfos = function (aSelection, callback) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.getListInfos');
    this.complementDataOnSelection({
        aSelection: aSelection,
        dataType: 'list',
        callback: callback
    });
};

/**
 * Download the form infos in the selection, if the form infos is already present it will not downloaded
 * @param {array} aSelection
 * @param {function} callback
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.getFormInfos = function (aSelection, callback) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.getFormInfos');
    this.complementDataOnSelection({
        aSelection: aSelection,
        dataType: 'form',
        callback: callback
    });
};

/**
 * Complete data on the dataType param off the selection, the request is maked by the business_object_id. If the param already exists the data is not erased
 * @param {object} opt_options
 * @param {array} opt_options.aSelection
 * @param {string} opt_options.dataType summary|list|form
 * @param {object} opt_options.complementParams params to add on the http request
 * @param {function} opt_options.callback
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.complementDataOnSelection = function (opt_options) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.complementDataOnSelection');

    var aSelection = goog.isDef(opt_options.aSelection) ? opt_options.aSelection : [];
    var dataType = goog.isDef(opt_options.dataType) ? opt_options.dataType : null;
    var complementParams = goog.isDef(opt_options.complementParams) ? opt_options.complementParams : {};
    var callback = goog.isDef(opt_options.callback) ? opt_options.callback : function () {
        return 0;
    };

    // Récupère la liste des bo
    var aBusinessObjects = [];
    for (var i = 0; i < aSelection.length; i++) {
        goog.array.binaryInsert(aBusinessObjects, aSelection[i]['bo_type']);
    }

    // Met à jour le compteur de requêtes
    this.requestCounter_ = goog.isNumber(this.requestCounter_) ? this.requestCounter_ : 0;
    this.requestCounter_ += aBusinessObjects.length;

    if (aBusinessObjects.length > 0)
        showAjaxLoader();

    var tmpSelection = [];

    for (var i = 0; i < aBusinessObjects.length; i++) {
        goog.array.clear(tmpSelection);

        // Récupère la liste des selections par couche
        for (var ii = 0; ii < aSelection.length; ii++) {
            if (aSelection[ii]['bo_type'] === aBusinessObjects[i])
                tmpSelection.push(aSelection[ii]);
        }

        this.complementDataOnSelection2_({
            aSelection: goog.array.clone(tmpSelection),
            bo_type: aBusinessObjects[i],
            dataType: dataType,
            complementParams: complementParams,
            callback: callback
        });

    }
};

/**
 * Add the data on the dataType param off the selection, if the param already exists the data is not erased
 * @param {object} opt_options
 * @param {array} opt_options.aSelection
 * @param {string} opt_options.bo_type
 * @param {string} opt_options.dataType summary|list|form
 * @param {object} opt_options.complementParams params to add on the http request
 * @param {function} opt_options.callback
 * @private
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.complementDataOnSelection2_ = function (opt_options) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.complementDataOnSelection2_');

    var aSelection = goog.isDef(opt_options.aSelection) ? opt_options.aSelection : [];
    var dataType = goog.isDef(opt_options.dataType) ? opt_options.dataType : null;
    var complementParams = goog.isDef(opt_options.complementParams) ? opt_options.complementParams : {};
    var callback = goog.isDef(opt_options.callback) ? opt_options.callback : function () {
        return 0;
    };

    var this_ = this;

    // Construction du filtre
    var filter = {
        'relation': 'OR',
        'operators': []
    };
    for (var i = 0; i < aSelection.length; i++) {
        if (!goog.isDef(aSelection[i]['bo_' + dataType])) {
            filter['operators'].push({
                'column': aSelection[i]['bo_id_field'].replace(/\./g, '"."'),
                'compare_operator': '=',
                'value': aSelection[i]['bo_id_value']
            });
        }
    }

    // Projection en cours
    var proj = this['map'].getView().getProjection().getCode().substring(5);

    // url de la requête
    if (goog.isDef(opt_options.bo_type)) {
        var url = oVmap['properties']['api_url'] + '/vmap/querys/' + opt_options.bo_type + '/' + dataType;
    } else {
        console.error('opt_options.bo_type not defined');
        return 0;
    }

    var params = {
        'result_srid': proj,
        'limit': 200
    };

    // Si on demande un type summary, alors on demande la geom
    if (dataType === 'summary') {
        params['get_geom'] = true;
        params['get_image'] = true;
    }

    // Ajoute si besoin les paramètres compémentaires
    goog.object.extend(params, complementParams);

    if (filter['operators'].length > 0) {
        params['filter'] = JSON.stringify(filter);

        ajaxRequest({
            'method': 'POST',
            'url': url,
            'headers': {
                'X-HTTP-Method-Override': 'GET',
                'Accept': 'application/x-vm-json'
            },
            'data': params,
            'scope': this.$scope_,
            'success': function (response) {

                var data = [];

                // Décrémente le compteur de requêtes
                this_.requestCounter_--;

                // Si le compteur de requêtes vaut zéro, alors on remet le curceur normal
                if (this_.requestCounter_ === 0) {
                    $('body').css({"cursor": ""});
                    hideAjaxLoader();
                }

                // Vérifie si il y a une erreur
                if (goog.isDef(response['data'])) {
                    if (goog.isDef(response['data']['errorMessage'])) {
                        bootbox.alert(response['data']['errorMessage']);
                    }
                }

                // Vérifie si il y a des données à afficher
                if (!goog.isDef(response['data']['data'])) {
                    oVmap.log('Pas de données à afficher');
                } else {
                    var data = response['data']['data'];
                }

                // Vérifie si la limite de données à afficher n'a pas été dépassée
                if (data.length > 199) {
                    var sAlert = '<h4>Attention, vous avez atteint la limite d\'objects affichables sur la carte</h4>';
                    bootbox.alert(sAlert);
                }

                // Ajoute le résultat de la requête
                this_.addQueryResult({
                    data: data,
                    selection: aSelection,
                    dataType: dataType,
                    callback: callback
                });
            },
            'error': function (response) {
                // Décrémente le compteur de requêtes
                this_.requestCounter_--;

                // Si le compteur de requêtes vaut zéro, alors on remet le curceur normal
                if (this_.requestCounter_ === 0) {
                    $('body').css({"cursor": ""});
                    hideAjaxLoader();
                    callback.call(this, aSelection);
                }
            }
        });
    } else {
        // Décrémente le compteur de requêtes
        this_.requestCounter_--;

        // Si le compteur de requêtes vaut zéro, alors on remet le curceur normal
        if (this_.requestCounter_ === 0) {
            $('body').css({"cursor": ""});
            hideAjaxLoader();
            callback.call(this, aSelection);
        }
    }
};

/**
 * Add the query result to this['aSelections'] removing the duplicate selections
 * @param {object} opt_options
 * @param {array<object>} opt_options.data
 * @param {array<object>} opt_options.selection
 * @param {string} opt_options.dataType summary|list|form
 * @param {function} opt_options.callback callback to run at the end of the function
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.addQueryResult = function (opt_options) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.addQueryResult');

    var aData = goog.isDef(opt_options.data) ? opt_options.data : [];
    var aSelections = goog.isDef(opt_options.selection) ? opt_options.selection : [];
    var dataType = goog.isDef(opt_options.dataType) ? opt_options.dataType : null;
    var callback = goog.isDef(opt_options.callback) ? opt_options.callback : function () {
        return 0;
    };

    // Formatage du résultat
    var aResult = [];
    for (var i = 0; i < aData.length; i++) {
        aResult.push(aData[i]);
        if (goog.isDef(aData[i]['bo_intersect_geom'])) {
            aResult[i]['olFeature'] = this.extractFeatureFromData(aData[i]);
        }
    }

    // Ajoute les résultats
    goog.array.insertArrayAt(aSelections, aResult, -1);

    // supprime les doublons
    aSelections = this.removeDuplicateSelections(aSelections);

    callback.call(this, aSelections);
};

/**
 * Remove the duplicate values without taking care of the 'loked' item
 * @param {array} aSelections
 * @returns {array}
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.removeDuplicateSelections = function (aSelections) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.removeDuplicateSelections');

    var itemsToRemove = [];

    for (var i = 0; i < aSelections.length; i++) {

        goog.array.clear(itemsToRemove);

        // Mémorise les élements à supprimer
        for (var ii = i + 1; ii < aSelections.length; ii++) {

            // Compare les objets
            //            if (angular.equals(aSelections[i]['bo_id_field'], aSelections[ii]['bo_id_field']) &&
            //                    angular.equals(aSelections[i]['bo_id_value'], aSelections[ii]['bo_id_value']) &&
            //                    angular.equals(aSelections[i]['bo_type'], aSelections[ii]['bo_type'])) {

            if (aSelections[i]['bo_id_field'] == aSelections[ii]['bo_id_field'] &&
                    aSelections[i]['bo_type'] == aSelections[ii]['bo_type'] &&
                    aSelections[i]['bo_id_value'] == aSelections[ii]['bo_id_value']) {

                itemsToRemove.push(ii);

                // Si le deuxième est locked, alors le premier passe locked (le deuxième sera supprimé dans tous les cas)
                if (aSelections[ii]['locked'] === true)
                    aSelections[i]['locked'] = true;

                // Paramètres à sauvegarder si une des deux l'a
                var paramsToSave = ['olFeature', 'mapPopup', 'bo_image_path', 'bo_summary', 'bo_summary_attributs', 'bo_list', 'bo_list_attributs', 'bo_form', 'bo_form_attributs', 'bo_json_form', 'bo_json_form_js', 'bo_json_form_css'];

                for (var iii = 0; iii < paramsToSave.length; iii++) {

                    // Si le deuxième a le paramètre, alors le premier le prend et vice versa
                    if (goog.isDef(aSelections[ii][paramsToSave[iii]]) && !goog.isDef(aSelections[i][paramsToSave[iii]]))
                        aSelections[i][paramsToSave[iii]] = aSelections[ii][paramsToSave[iii]];
                    if (goog.isDef(aSelections[i][paramsToSave[iii]]) && !goog.isDef(aSelections[ii][paramsToSave[iii]]))
                        aSelections[ii][paramsToSave[iii]] = aSelections[i][paramsToSave[iii]];
                }
            }
        }

        // Supprime les doublons
        for (var j = 0; j < itemsToRemove.length; j++) {
            goog.array.removeAt(aSelections, itemsToRemove[j]);
        }
    }

    return aSelections;
};

/**
 * Update on database the BO values
 * @param {string} bo_type
 * @param {object} boValues
 * @param {function} successCallback
 * @param {function} errorCallback
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.updateBOValues = function (bo_type, boValues, successCallback, errorCallback) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.updateBOValues');

    if (!goog.isDef(bo_type))
        return 0;

    boValues = goog.isDef(boValues) ? boValues : {};
    successCallback = goog.isDef(successCallback) ? successCallback : angular.noop;
    errorCallback = goog.isDef(errorCallback) ? errorCallback : angular.noop;

    var data = this.getFormDataFromValues(boValues);
    var aLayers = this['map'].getLayers().getArray();

    ajaxRequest({
        'method': 'PUT',
        'url': oVmap['properties']['api_url'] + '/vmap/querys/' + bo_type,
        'headers': {
            'Accept': 'application/x-vm-json',
        },
        'data': data,
        'scope': this.$scope_,
        'success': function (response) {

            if (!goog.isDef(response['data'])) {
                console.error('Aucune valeur retournée');
                errorCallback.call();
                return 0;
            }

            if (goog.isDef(response['data']['errorMessage'])) {
                console.error(response['data']['errorMessage']);
                errorCallback.call();
                return 0;
            }

            // recharge les couches impliquées
            for (var i = 0; i < aLayers.length; i++) {
                var aBusinessObjects = aLayers[i].getProperties()['business_objects'];
                if (goog.isArray(aBusinessObjects)) {
                    for (var ii = 0; ii < aBusinessObjects.length; ii++) {
                        if (aBusinessObjects[ii]['business_object_id'] === bo_type) {
                            aLayers[i].refreshWithTimestamp();
                        }
                    }
                }
            }

            // Message ok
            $.notify("Opération réalisée avec succès, les couches de la carte se mettent à jour", "info");

            successCallback.call();
        }
    });
};

/**
 * Get the form values to send
 * @param {object} oFormValues
 * @param {object} oFormDefinition
 * @param {string} sFormDefinitionName
 * @param {array|undefined} aParamsToSave
 * @returns {object|null}
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.getFormData = function (oFormValues, oFormDefinition, sFormDefinitionName, aParamsToSave) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.getFormData');

    if (!goog.isDefAndNotNull(oFormValues)) {
        console.error('oFormValues is undefined or null');
        return null;
    }
    if (!goog.isDefAndNotNull(oFormDefinition)) {
        console.error('oFormDefinition is undefined or null');
        return null;
    }
    if (!goog.isDefAndNotNull(sFormDefinitionName)) {
        console.error('sFormDefinitionName is undefined or null');
        return null;
    }
    if (!goog.isDefAndNotNull(aParamsToSave)) {
        aParamsToSave = [];
    }

    var envSrvc = angular.element(vitisApp.appMainDrtv).injector().get(["envSrvc"]);
    var formSrvc = angular.element(vitisApp.appMainDrtv).injector().get(["formSrvc"]);

    envSrvc["oFormValues"] = oFormValues;
    envSrvc["oFormDefinition"] = oFormDefinition;
    envSrvc["sFormDefinitionName"] = sFormDefinitionName;

    var oFormData = formSrvc['getFormData'](sFormDefinitionName, true);

    for (var i = 0; i < aParamsToSave.length; i++) {
        if (goog.isDefAndNotNull(oFormValues[sFormDefinitionName][aParamsToSave[i]])) {
            oFormData[aParamsToSave[i]] = oFormValues[sFormDefinitionName][aParamsToSave[i]];
        }
    }

    return oFormData;
};

/**
 * Creates a FormData object from oValues
 * @param {object} oValues
 * @returns {FormData}
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.getFormDataFromValues = function (oValues) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.getFormDataFromValues');

    var formSrvc = angular.element(vitisApp.appMainDrtv).injector().get(["formSrvc"]);
    return formSrvc['getFormDataFromValues'](oValues);
};

nsVmap.nsToolsManager.Select.prototype.selectController.prototype.getUnchangedFilesFromValues = function (oValues) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.getUnchangedFilesFromValues');

    var oUnchangedFiles = {};
    for (var key in oValues) {

        var bIsMultipleFiles = false;
        if (goog.isObject(oValues[key])) {
            if (goog.isDefAndNotNull(oValues[key].length)) {
                bIsMultipleFiles = true
            }
        }

        // Fichier ?
        if (bIsMultipleFiles) {
            for (var i = 0; i < oValues[key].length; i++) {
                if (oValues[key][i]._modified === false) {

                    if (!goog.isArray(oUnchangedFiles[key])) {
                        oUnchangedFiles[key] = [];
                    }

                    oUnchangedFiles[key].push(oValues[key][i]['name']);

                }
            }
        }
    }

    return oUnchangedFiles;
};

/**
 * Suggest if realy want to delete the object and call submitDeleteObject
 * @param {object} selection
 * @param {object} selection.bo_type
 * @param {object} selection.bo_id_value
 * @param {function} callback
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.deleteObject = function (selection, callback) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.deleteObject');

    var this_ = this;

    bootbox.confirm("<h4>Supprimer définitivement cet objet de la base de données ?</h4>", function (result) {
        if (result === true)
            this_.submitDeleteObject(selection, callback);
    });

};

/**
 * Suggest if realy want to delete the objects and call submitDeleteObject
 * @param {array<object>} aSelection
 * @param {function} callback
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.deleteMultipleObjects = function (aSelection, callback) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.deleteMultipleObjects');

    var this_ = this;

    bootbox.confirm("<h4>Supprimer définitivement ces objets de la base de données ?</h4>", function (result) {
        if (result === true) {
            for (var i = 0; i < aSelection.length; i++) {
                this_.submitDeleteObject(aSelection[i], callback);
            }
        }
    });

};

/**
 * Submit a DELETE request to the server for delete the object from the datasource
 * @param {object} selection
 * @param {object} selection.bo_type
 * @param {object} selection.bo_id_value
 * @param {function} callback
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.submitDeleteObject = function (selection, callback) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.submitDeleteObject');

    var boId = selection['bo_type'];
    var boIdValue = selection['bo_id_value'];
    var aLayers = this['map'].getLayers().getArray();

    ajaxRequest({
        'method': 'DELETE',
        'url': oVmap['properties']['api_url'] + '/vmap/querys/' + boId,
        'headers': {
            'Accept': 'application/x-vm-json'
        },
        'params': {
            'idList': boIdValue
        },
        'scope': this.$scope_,
        'success': function (response) {

            // recharge les couches impliquées
            for (var i = 0; i < aLayers.length; i++) {
                var aBusinessObjects = aLayers[i].getProperties()['business_objects'];
                if (goog.isArray(aBusinessObjects)) {
                    for (var ii = 0; ii < aBusinessObjects.length; ii++) {
                        if (aBusinessObjects[ii]['business_object_id'] === boId) {
                            aLayers[i].refreshWithTimestamp();
                        }
                    }
                }
            }

            // Message ok
            $.notify("Opération réalisée avec succès, les couches de la carte se mettent à jour", "info");

            // Callback
            callback.call();
        },
        'error': function (response) {

        }
    });
};

/**
 * Add the selected elements from the table to the card
 * @param {array} aSelection
 * @param {boolean} bReplace true to empty the card before adding items, default false
 * @param {boolean|undefined} bZoomOnFeatures true to zoom on features, default true
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.addToCard = function (aSelection, bReplace, bZoomOnFeatures) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.addToCard');

    console.log('bZoomOnFeatures', bZoomOnFeatures);

    bReplace = bReplace === true ? true : false;
    bZoomOnFeatures = bZoomOnFeatures === false ? false : true;

    // Vide le panier
    if (bReplace) {
        oVmap.getToolsManager().getInfoContainer().removeAll();
    }

    var this_ = this;

    // Télécharge les features de la sélection
    setTimeout(function () {
        this_.getSummaryInfos(aSelection, function (aSelection) {
            this_.getListInfos(aSelection, function (aSelection) {

                if (!goog.isDef(aSelection) || aSelection.length <= 0) {
                    console.error('aSelection empty or inexistent');
                    return 0;
                }

                var aData = [];
                var bo_type = aSelection[0]['bo_type'];
                var bo_title = aSelection[0]['bo_title'];

                var rowOptions = {
                    invisibleColumns: ['selection', 'state']
                };

                // Extrait la data (qui cette fois ci contient la feature)
                for (var i = 0; i < aSelection.length; i++) {
                    if (!goog.isDefAndNotNull(aSelection[i]['bo_list'])) {
                        console.error("Selection[i]['bo_list'] undefined");
                        continue;
                    }
                    if (!goog.isDefAndNotNull(aSelection[i]['olFeature'])) {
                        console.error("Selection[i]['olFeature'] undefined");
                        continue;
                    }
                    aData.push(aSelection[i]['bo_list']);
                    aData[i]['feature'] = aSelection[i]['olFeature'];

                    // Si ne contient pas le champ identifiant
                    if (!goog.isDefAndNotNull(aData[i][aSelection[i]['bo_id_field']])) {
                        aData[i][aSelection[i]['bo_id_field']] = aSelection[i]['bo_id_value'];
                        rowOptions.invisibleColumns.push(aSelection[i]['bo_id_field']);
                    }
                }

                // Crée l'onglet correspondant
                if (oVmap.getToolsManager().getInfoContainer().getTabByCode(bo_type) === undefined)
                    oVmap.getToolsManager().getInfoContainer().addTab({tabCode: bo_type, tabName: bo_title, actions: ['zoom', 'delete']});

                var tabOptions = {tabCode: bo_type};
                rowOptions.data = aData;

                // Ajoute les inos au panier
                oVmap.getToolsManager().getInfoContainer().addMultipleRows(tabOptions, rowOptions);

                // Ferme la modale
                $('#select-list-modal').modal('hide');

                // Affiche le panier
                oVmap.getToolsManager().getInfoContainer().showBar();

                // Affiche les features
                oVmap.getToolsManager().getInfoContainer().displayTabFeatures(bo_type);

                console.log('bZoomOnFeatures', bZoomOnFeatures);

                // Foom sur les features
                if (bZoomOnFeatures) {
                    setTimeout(function () {
                        oVmap.getToolsManager().getInfoContainer().zoomOnTabFeatures(bo_type);
                    }, 500);
                }
            });
        });
    });
};

/**
 * Parse the geom title and replace the {{ }}
 * @param {string} sFormTitle
 * @param {object} oScope
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.parseFormTile = function (sFormTitle, oScope) {

    if (!goog.isString(sFormTitle)) {
        return sFormTitle;
    }
    if (!goog.isObject(oScope)) {
        return sFormTitle;
    }

    var sAttr;
    if (sFormTitle.search('{{') !== -1 && sFormTitle.search('}}') !== -1) {
        sAttr = sFormTitle.substr(sFormTitle.search('{{') + 2, sFormTitle.search('}}') - sFormTitle.search('{{') - 2);
        if (goog.isDefAndNotNull(oScope[sAttr])) {
            sFormTitle = sFormTitle.replace('{{' + sAttr + '}}', oScope[sAttr]);
        }
    }
    if (sFormTitle.search('{{') !== -1 && sFormTitle.search('}}') !== -1) {
        this.parseFormTile(sFormTitle, oScope);
    }

    sFormTitle = $('<div/>').html(oVmap['$sanitize'](sFormTitle)).text();

    return sFormTitle;
};



// Édition de la géométrie

/**
 * Edit the form feature
 * @param {object} selection
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.editFormFeature = function (selection) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.editFormFeature');

    // Ferme le formulaire d'édition
    $('#select-edit-modal').modal('hide');

    // Ferme les popup
    this.getBasicSelectScope()['ctrl'].removePopups();

    // Ferme le requêteur
    $('#select-list-modal').modal('hide');

    // Lance l'édition
    this.editFeature(selection, true);
};

/**
 * Start the geometry edition
 * @param {object} selection
 * @param {boolean} bFromEditForm true if the request comes to the EditForm
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.editFeature = function (selection, bFromEditForm) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.editFeature');

    var this_ = this;

    var oBusinessObjects = oVmap.getMapManager().getQueryableBusinessObjects();

    if (!goog.isDefAndNotNull(selection['bo_type'])) {
        console.error("selection['bo_type'] not defined");
        return 0;
    }
    if (!goog.isDefAndNotNull(oBusinessObjects[selection['bo_type']])) {
        console.error("oBusinessObjects[selection['bo_type']] not defined");
        return 0;
    }
    if (!goog.isDefAndNotNull(oBusinessObjects[selection['bo_type']]['bo_geom_type'])) {
        console.error("oBusinessObjects[selection['bo_type']]['bo_geom_type'] not defined");
        return 0;
    }

    this['editableSelection'] = selection;
    this['editableFeatureType'] = oBusinessObjects[selection['bo_type']]['bo_geom_type'];

    this['resetValuesFormForInsertGeom']();

    if (this['editableFeatureType'] === 'GEOMETRYCOLLECTION') {
        this['addPartGeomType'] = '';
    } else if (this['editableFeatureType'] === 'POLYGON' || this['editableFeatureType'] === 'MULTIPOLYGON') {
        this['addPartGeomType'] = 'Polygon';
        this_['onlySelectWKT'] = true;
    } else if (this['editableFeatureType'] === 'LINESTRING' || this['editableFeatureType'] === 'MULTILINESTRING') {
        this['addPartGeomType'] = 'LineString';
        this_['onlySelectWKT'] = true;
    } else if (this['editableFeatureType'] === 'POINT' || this['editableFeatureType'] === 'MULTIPOINT') {
        this['addPartGeomType'] = 'Point';
        if (this['editableFeatureType'] === 'POINT') {
            this_['selectTypeInsertGeom'] = 'XY';
        }
    }

    this['objectCanMove'] = false;

    if (bFromEditForm) {
        this['editGeometryFromForm'] = true;
    } else {
        this['editGeometryFromForm'] = false;
    }

    if (oVmap['properties']['is_mobile']) {
        oVmap.getMap().startMobileGeomEdition(this_['editableSelection']['olFeature'], function (olFeature) {
            setTimeout(function () {
                this_['editableSelection']['olFeature'] = olFeature;
                this_.submitGeomEdition();
            });
        });
    } else {
        // Affiche la palette de modification
        $('#basictools-select-modify-palette').show();

        // Chage le liste des BO intersectables par calcul de polygones jointifs
        this_.loadAvoidSuperpositionBos();

        // Timeout car lors du toggleOutTools, clearOverlays est lancé
        setTimeout(function () {
            // Ajoute la/les feature(s)
            if (goog.isDefAndNotNull(this_['editableSelection']['olFeature'])) {
                this_.putElementFeatureOnOverlay();
            }

            setTimeout(function () {
                // Ajoute l'interraction
                if (this_.oOverlayLayer_.getSource().getFeaturesCollection().getLength() > 0) {
                    this_.startEdition('editFeature', this_['currentAction'] === 'basicTools-select-editFeature');
                }
            }, 100);
        }, 500);
    }
};

/**
 * Set the drawTool
 * @param {string} type (editFeature, deleteFeature, addHole, moveFeature)
 * @param {boolean} isActive true if the selection is already active
 * @returns {Number}
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.startEdition = function (type, isActive) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.startEdition');

    var this_ = this;

    this.cancelEdition();

    if (isActive === true)
        return 0;

    if (type.substr(0, 6) === 'insert') {

        //Ajoute la tooltip
        oVmap.getMap().getMapTooltip().displayMessage('Cliquez pour commencer à dessiner');

        // Ajoute l'interraction
        this.startDrawing(type.substr(6));

    } else if (type === 'editFeature') {

        //Ajoute la tooltip
        oVmap.getMap().getMapTooltip().displayMessage('Cliquez sur la géométrie pour la modifier');

        // Ajoute l'interraction
        oVmap.getMap().setInteraction(this.modify_, 'basicTools-select-' + type);

        this_.loadVectorSnappingData();

    } else if (type === 'moveFeature') {

        //Ajoute la tooltip
        oVmap.getMap().getMapTooltip().displayMessage('Cliquez sur la géométrie et rester appuyé pour la déplacer');

        // Ajoute l'interraction
        oVmap.getMap().setInteraction(this.drag_, 'basicTools-select-' + type);

        this_.oSnapOverlayLayer_.setVisible(false);

    } else if (type === 'deleteFeature') {

        //Ajoute la tooltip
        oVmap.getMap().getMapTooltip().displayMessage('Cliquez sur la géométrie pour la supprimer');

        // Ajoute l'interraction
        oVmap.getMap().setInteraction(this.delete_, 'basicTools-select-' + type);

        this_.oSnapOverlayLayer_.setVisible(false);

    } else if (type === 'addHole') {

        //Ajoute la tooltip
        oVmap.getMap().getMapTooltip().displayMessage('Dessinez un polygone à l\'intérieur d\'un autre pour créer un trou');

        // Ajoute l'interraction (reliée à aucune featureCollection)
        this.draw_ = oVmap.getMap().setDrawInteraction({
            type: 'Polygon'
        }, 'basicTools-select-' + type);

        // Copie les features présentes avant le dessin
        var aFeaturesCopy = goog.array.clone(this.oOverlayFeatures_.getArray());

        this.draw_.on('drawend',
        function (evt) {
            // Récupère la feature ajoutée
            var feature = evt.feature;
            var linearRing = feature.getGeometry().getLinearRing(0);
            var bIsContained = false;
            for (var i = 0; i < aFeaturesCopy.length; i++) {
                // Si le trou est contenu dans une des features
                if (aFeaturesCopy[i].getGeometry().containsPolygon(feature.getGeometry())) {
                    // Troue le polygone
                    aFeaturesCopy[i].getGeometry().appendLinearRing(linearRing);
                    bIsContained = true;
                }
            }

            if (!bIsContained) {
                bootbox.alert('<h4>Trou non valide: il doit être contenu dans un polygone dessiné</h4>');
            } else {
                setTimeout(function () {
                    this_.putFeaturesOnTheElement(this_.oOverlayFeatures_.getArray());
                });
            }
        }, this);

    } else if (type === 'deleteHole') {

        //Ajoute la tooltip
        oVmap.getMap().getMapTooltip().displayMessage("Cliquez dans un trou pour le supprimer");

        // Ajoute les trous dans la couche oOverlayHolesLayer_
        this.addHolesIntoHolesLayer();

        // Ajoute l'interraction de suppresion des trous
        oVmap.getMap().setInteraction(this.deleteHole_, 'basicTools-select-' + type);

    } else {
        console.error('startEdition: type (' + type + ') not allowed');
        return 0;
    }
};

/**
 * Cancel the edition
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.cancelEdition = function () {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.cancelEdition');

    // Supprime les interractions
    oVmap.getMap().removeActionsOnMap();

    // Cache la tooltip
    oVmap.getMap().getMapTooltip().hide();

    // vide oOverlayHolesLayer_
    this.oOverlayHolesLayer_.getSource().clear();
};

/**
 * This function add the holes contained into oOverlayLayer_ to oOverlayHolesLayer_
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.addHolesIntoHolesLayer = function () {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.addHolesIntoHolesLayer');

    var aFeatures = this.oOverlayFeatures_.getArray();

    this.oOverlayHolesLayer_.getSource().clear();

    for (var i = 0; i < aFeatures.length; i++) {
        var aLinearRings = aFeatures[i].getGeometry().getLinearRings();
        if (aLinearRings.length > 1) {

            for (var ii = 1; ii < aLinearRings.length; ii++) {

                var newPolygon = new ol.geom.Polygon();
                newPolygon.setCoordinates([aLinearRings[ii].getCoordinates()]);

                this.oOverlayHolesLayer_.getSource().addFeature(new ol.Feature({
                    geometry: newPolygon
                }));
            }
        }
    }
};

/**
 * Delete the selected hole
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.deleteHole = function () {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.deleteHole');

    var this_ = this;

    /**
     * Holes selected by deleteHole_
     * @type array
     */
    var aSelectedHoles = this.deleteHole_.getFeatures().getArray();

    if (aSelectedHoles.length > 0) {
        for (var i = 0; i < aSelectedHoles.length; i++) {

            /**
             * Selected hole
             * @type ol.geom.polygon
             */
            var hole = aSelectedHoles[i].getGeometry();

            var bHoleFounded = false;

            // Compare pour chaque polygone si il a un trou ayant les mêmes coordonnées que hole
            var aFeatures = this.oOverlayFeatures_.getArray();
            for (var ii = 0; ii < aFeatures.length; ii++) {
                var aHolesCoords = aFeatures[ii].getGeometry().getCoordinates();
                for (var iii = aHolesCoords.length - 1; iii >= 0; iii--) {
                    if (angular.equals(hole.getCoordinates()[0], aHolesCoords[iii])) {
                        aHolesCoords.splice(iii, 1);
                        bHoleFounded = true;
                    }
                }
                aFeatures[ii].getGeometry().setCoordinates(aHolesCoords);
            }
            if (bHoleFounded)
                this.oOverlayHolesLayer_.getSource().removeFeature(aSelectedHoles[i]);
        }

        this.deleteHole_.getFeatures().clear();


        setTimeout(function () {
            this_.putFeaturesOnTheElement(this_.oOverlayFeatures_.getArray());
        });
    }
};

/**
 * Start the drawing experience
 * @param {ol.geom} type geometry type
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.startDrawing = function (type) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.startDrawing');

    var this_ = this;

    // Lance setDrawInteraction avec this.oOverlayFeatures_ comme cible pour les features
    this.draw_ = oVmap.getMap().setDrawInteraction({
        type: type,
        features: this.oOverlayFeatures_
    }, 'basicTools-select-insert' + type);
    this.draw_.on('drawend', function (evt) {

        // Récupère la feature ajoutée
        var feature = evt.feature;

        setTimeout(function () {

            // Transforme les cercles en polygones de 32 côtés
            // car ils ne sont pas acceptés par WKT
            if (type === 'Circle') {
                var circleGeom = feature.getGeometry();
                var polygonGeom = new ol.geom.Polygon.fromCircle(circleGeom);

                // Supprime l'ancienne feature de l'overlay
                this_.oOverlayLayer_.getSource().removeFeature(feature);

                feature = new ol.Feature({
                    geometry: polygonGeom
                });

                // Ajoute la nouvelle feature à l'overlay
                this_.oOverlayLayer_.getSource().addFeature(feature);
            }

            // Évite les superpositions avec les autres couches si besoin
            this_.avoidSuperpositions(feature);

            // Ajoute les géométries en mode multi ou simple à editableSelection.olFeature
            this_.putFeaturesOnTheElement(this_.oOverlayFeatures_.getArray());
        });
    }, this);
};

/**
 * Edit the given feature to avoid superpositions with layers defined in snapOptions.avoidSuperpositions
 * @param {ol.Feature} olFeature
 * @returns {ol.Feature}
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.avoidSuperpositions = function (olFeature) {
    oVmap.log('nsVmap.nsToolsManager.Select.selectController.avoidSuperpositions');

    var deferred = this.$q_.defer();

    // Objets métiers à intersecter
    var aSuperpositionBos = [];
    for (var bo_id in this['snapOptions']['avoidSuperpositions']) {
        if (this['snapOptions']['avoidSuperpositions'][bo_id] === true) {
            aSuperpositionBos.push(bo_id);
        }
    }

    // Remplace la géométrie par celle calculée
    if (aSuperpositionBos.length > 0) {
        this.setDiffGeom_(aSuperpositionBos, olFeature).then(function(olFeature){
            deferred.resolve(olFeature);
        });
    } else {
        deferred.resolve(olFeature);
    }

    return deferred.promise;
}

/**
 * Modify the feature geometry to avoid superpositions with BOs in aSuperpositionBos
 *
 * @param  {string} aSuperpositionBos
 * @param  {ol.Feature} olFeature
 * @param  {object|undefines} opt_options
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.setDiffGeom_ = function (aSuperpositionBos, olFeature, opt_options) {
    oVmap.log('nsVmap.nsToolsManager.Select.selectController.setDiffGeom_');

    if (!goog.isArray(aSuperpositionBos)) {
        return 0;
    }
    if (!goog.isDefAndNotNull(olFeature)) {
        return 0;
    }
    if (!goog.isDefAndNotNull(opt_options)) {
        opt_options = {
            bo_index: 0
        }
    }

    var this_ = this;
    var deferred = this.$q_.defer();
    var sEWKTGeom = oVmap.getEWKTFromGeom(olFeature.getGeometry());

    this.getDiffGeom_(aSuperpositionBos[opt_options.bo_index], sEWKTGeom).then(function(sNewEWKTGeom){

        // Remplace la géométrie
        olFeature.setGeometry(oVmap.getGeomFromEWKT(sNewEWKTGeom));

        // Relance la fonction pour les autres objets métiers
        opt_options.bo_index ++;
        if (opt_options.bo_index < aSuperpositionBos.length) {
            this_.setDiffGeom_(aSuperpositionBos, olFeature, opt_options).then(function(){
                deferred.resolve(olFeature);
            }, function(error){
                console.error(error);
            });
        } else {
            deferred.resolve(olFeature);
        }
    }, function(err){
        console.error('Cannot get the diff geom: ', err)
    });

    return deferred.promise;
}

/**
 * Get the diff superposition geometry
 *
 * @param  {string} sBoId
 * @param  {string} sEWKTGeom
 * @return {promise}
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.getDiffGeom_ = function (sBoId, sEWKTGeom) {
    oVmap.log('nsVmap.nsToolsManager.Select.selectController.getDiffGeom_');

    var oParams = {
        'intersect_geom': sEWKTGeom
    };

    // Filtre pour ne pas s'éviter lui même
    if (goog.isDefAndNotNull(this['editableSelection'])) {
        if (this['editableSelection']['bo_type'] === sBoId) {
            if (goog.isDefAndNotNull(this['editableSelection']['bo_id_field']) &&
                goog.isDefAndNotNull(this['editableSelection']['bo_id_value'])) {

                oParams['filter'] = {
                    'column': this['editableSelection']['bo_id_field'],
                    'compare_operator': '!=',
                    'value': this['editableSelection']['bo_id_value']
                }
            }
        }
    }

    var deferred = this.$q_.defer();
    ajaxRequest({
        'method': 'POST',
        'url': oVmap['properties']['api_url'] + '/vmap/querys/' + sBoId + '/diff_geometry',
        'headers': {
            'X-HTTP-Method-Override': 'GET',
            'Accept': 'application/x-vm-json'
        },
        'data': oParams,
        'scope': this.$scope_,
        'success': function (response) {

            if (!goog.isDefAndNotNull(response['data'])) {
                deferred.reject('response.data not defined');
                return 0;
            }
            if (goog.isDefAndNotNull(response['data']['errorMessage'])) {
                deferred.reject(response['data']['errorMessage']);
                return 0;
            }
            if (!goog.isDefAndNotNull(response['data'][0])) {
                deferred.resolve(sEWKTGeom);
                return 0;
            }
            if (!goog.isDefAndNotNull(response['data'][0]['diff_geom'])) {
                deferred.resolve(sEWKTGeom);
                return 0;
            }
            deferred.resolve(response['data'][0]['diff_geom']);
        },
        'error': function (response) {
            deferred.reject(response);
        }
    });
    return deferred.promise;
}

/**
 * Put the array of features passed on aFeatures on editableSelection.olFeature with MULTI... form
 * @param {array} aFeatures
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.putFeaturesOnTheElement = function (aFeatures) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.putFeaturesOnTheElement');

    if (!goog.isDefAndNotNull(aFeatures)) {
        console.error('aFeatures undefined');
        return 0;
    }
    if (!goog.isArray(aFeatures)) {
        console.error('aFeatures is not an array');
        return 0;
    }

    // Cas de feature simple
    if (this['editableFeatureType'].substr(0, 5) !== 'MULTI' && this['editableFeatureType'] !== 'GEOMETRYCOLLECTION') {
        // La nouvelle feature devient la feature de référence
        this['editableSelection']['olFeature'] = aFeatures[aFeatures.length - 1];
        // Supprime les aurtes features
        for (var i = 0; i < aFeatures.length - 1; i++) {
            this.oOverlayLayer_.getSource().removeFeature(aFeatures[i]);
        }
    }
    // Cas de feature multiple
    else {
        if (this['editableFeatureType'] === 'MULTIPOLYGON') {

            var oMultiPolygon = new ol.geom.MultiPolygon();
            var aCoordinates = [];

            for (var i = 0; i < aFeatures.length; i++) {
                if (aFeatures[i].getGeometry().getType() === 'Polygon') {
                    aCoordinates.push(aFeatures[i].getGeometry().getCoordinates());
                }
            }

            oMultiPolygon.setCoordinates(aCoordinates);
            this['editableSelection']['olFeature'] = new ol.Feature({geometry: oMultiPolygon});

        }
        if (this['editableFeatureType'] === 'MULTILINESTRING') {

            var oMultiLineString = new ol.geom.MultiLineString();
            var aCoordinates = [];

            for (var i = 0; i < aFeatures.length; i++) {
                if (aFeatures[i].getGeometry().getType() === 'LineString') {
                    aCoordinates.push(aFeatures[i].getGeometry().getCoordinates());
                }
            }

            oMultiLineString.setCoordinates(aCoordinates);
            this['editableSelection']['olFeature'] = new ol.Feature({geometry: oMultiLineString});

        }
        if (this['editableFeatureType'] === 'MULTIPOINT') {

            var oMultiPoint = new ol.geom.MultiPoint();
            var aCoordinates = [];

            for (var i = 0; i < aFeatures.length; i++) {
                if (aFeatures[i].getGeometry().getType() === 'Point') {
                    aCoordinates.push(aFeatures[i].getGeometry().getCoordinates());
                }
            }

            oMultiPoint.setCoordinates(aCoordinates);
            this['editableSelection']['olFeature'] = new ol.Feature({geometry: oMultiPoint});
        }
    }
};

/**
 * Put the features in editableSelection.olFeature on the map (disabling the Multi.... form)
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.putElementFeatureOnOverlay = function () {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.putElementFeatureOnOverlay');

    if (!goog.isDefAndNotNull(this['editableSelection']['olFeature'])) {
        console.error("this['editableSelection']['olFeature'] not defined");
        return 0;
    }
    if (!goog.isDefAndNotNull(this['editableSelection']['olFeature'].getGeometry())) {
        console.error("this['editableSelection']['olFeature'].getGeometry() not defined");
        return 0;
    }

    // Cas de feature simple
    if (this['editableFeatureType'].substr(0, 5) !== 'MULTI' && this['editableFeatureType'] !== 'GEOMETRYCOLLECTION') {
        this.oOverlayLayer_.getSource().clear;
        this.oOverlayLayer_.getSource().addFeature(this['editableSelection']['olFeature']);
    }
    // Cas de feature multiple
    else {
        var aFeatures = [];
        var aGeoms = [];

        if (this['editableFeatureType'] === 'MULTIPOINT') {
            aGeoms = goog.array.clone(this['editableSelection']['olFeature'].getGeometry().getPoints());
        }
        if (this['editableFeatureType'] === 'MULTILINESTRING') {
            aGeoms = goog.array.clone(this['editableSelection']['olFeature'].getGeometry().getLineStrings());
        }
        if (this['editableFeatureType'] === 'MULTIPOLYGON') {
            aGeoms = goog.array.clone(this['editableSelection']['olFeature'].getGeometry().getPolygons());
        }

        for (var i = 0; i < aGeoms.length; i++) {
            aFeatures.push(new ol.Feature({
                geometry: aGeoms[i]
            }));
        }
        this.oOverlayLayer_.getSource().clear;
        this.oOverlayLayer_.getSource().addFeatures(aFeatures);
    }

};

/**
 * Submit the geometry modifications
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.submitGeomEdition = function () {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.submitGeomEdition');

    var oFormValues = {};
    oFormValues[this['editableSelection']['bo_id_field']] = this['editableSelection']['bo_id_value'];

    if (goog.isDefAndNotNull(this['editableSelection']['olFeature']))
        oFormValues[this['editableSelection']['geom_column']] = oVmap.getEWKTFromGeom(this['editableSelection']['olFeature'].getGeometry());
    else
        oFormValues[this['editableSelection']['geom_column']] = '';

    this.updateBOValues(this['editableSelection']['bo_type'], oFormValues, function () {
        oVmap.getToolsManager().getBasicTools().toggleOutTools();
    }, function () {
        oVmap.getToolsManager().getBasicTools().toggleOutTools();
    });
};

/**
 * Write the feature into the EditForm oFormValues and open it
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.validateGeomAndDisplayEditForm = function () {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.validateGeomAndDisplayEditForm');

    var this_ = this;

    var editFormReaderScope = this.getEditFormReaderScope();
    editFormReaderScope.$evalAsync(function () {

        if (goog.isDefAndNotNull(this_['editableSelection']['olFeature']))
            if (goog.isDefAndNotNull(this_['editableSelection']['olFeature'].getGeometry()))
                editFormReaderScope['oFormValues']['update'][this_['editableSelection']['geom_column']] = oVmap.getEWKTFromGeom(this_['editableSelection']['olFeature'].getGeometry());
        else
            editFormReaderScope['oFormValues']['update'][this_['editableSelection']['geom_column']] = '';
        else
            editFormReaderScope['oFormValues']['update'][this_['editableSelection']['geom_column']] = '';


        // Cache la palette de modification
        $('#basictools-select-modify-palette').hide();

        // Enlève les interractions
        oVmap.getMap().removeActionsAndTooltips();

        // Affiche le formulaire d'édition
        $('#select-edit-modal').modal('show');

        // Enlève la feature de la carte
        this_.clearOverlays();
    });
};

/**
 * Remove all was did during the geometry edition
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.clearOverlays = function () {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.clearOverlays');

    // Supprime la feature
    if (this.oOverlayLayer_.getSource().getFeaturesCollection().getLength() > 0) {
        this.oOverlayLayer_.getSource().clear();
    }
    if (this.oOverlayHolesLayer_.getSource().getFeaturesCollection().getLength() > 0) {
        this.oOverlayHolesLayer_.getSource().clear();
    }

    // Supprime le snapping
    this.oSnapOverlayLayer_.getSource().clear();
    oVmap.getMap().getOLMap().removeInteraction(this.snap_);

    // Supprime les points GetFeatureInfo
    this.oGetFeatureInfoOverlay_.getSource().clear();
};

/**
 * Remove all was did during the geometry edition
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.finishEdition = function () {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.finishEdition');

    // Supprime le snapping
    this.oSnapOverlayLayer_.getSource().clear();
    oVmap.getMap().getOLMap().removeInteraction(this.snap_);

    // Supprime la feature
    this.clearOverlays();

    // Cache la tooltip
    oVmap.getMap().getMapTooltip().hide();

    // cache la palette d'édition
    oVmap.getToolsManager().getBasicTools().toggleOutTools();
};


// Snapping

/**
 * Show the snapping options modal
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.showSnappingOptionsModal = function () {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.showSnappingOptionsModal');

    var this_ = this;
    this['tmpSnapOptions'] = angular.copy(this['snapOptions']);

    this.$scope_.$applyAsync(function(){
        for (var i = 0; i < this_['aQueryableBOs'].length; i++) {
            if (!goog.isDefAndNotNull(this_['tmpSnapOptions']['avoidSuperpositions'][this_['aQueryableBOs'][i]['bo_id']])) {
                this_['tmpSnapOptions']['avoidSuperpositions'][this_['aQueryableBOs'][i]['bo_id']] = false;
            }
        }
    });

    $('#vmap-select-snap-options-modal').modal('show');
};

/**
 * Validate and hide the snapping options modal
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.submitSnappingOptionsModal = function () {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.submitSnappingOptionsModal');

    this['snapOptions'] = angular.copy(this['tmpSnapOptions']);

    oVmap.getMap().getOLMap().removeInteraction(this.snap_);
    this.snap_ = new ol.interaction.Snap({
        vertex: true,
        edge: (this['snapOptions']['mode'] === 'segment_edge_node' ? true : false),
        pixelTolerance: this['snapOptions']['tolerance'],
        source: this.oSnapOverlayLayer_.getSource()
    });
    oVmap.getMap().getOLMap().addInteraction(this.snap_);

    $('#vmap-select-snap-options-modal').modal('hide');

    this.loadVectorSnappingData();
};

/**
 * Reset snapping option to default values
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.resetSnapOptions = function () {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.resetSnapOptions');

    if (goog.isDefAndNotNull(oVmap['properties']['snapping'])) {
        this['tmpSnapOptions']['tolerance'] = oVmap['properties']['snapping']['defaut_tolerance'];
        this['tmpSnapOptions']['mode'] = oVmap['properties']['snapping']['defaut_snapp_mode'];
        this['tmpSnapOptions']['limit'] = oVmap['properties']['snapping']['defaut_limit'];
        this['tmpSnapOptions']['visible'] = oVmap['properties']['snapping']['defaut_visibility'];
    }
    this.submitSnappingOptionsModal();
};

/**
 * Selector function to only reload object when needed
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.loadVectorSnappingData = function () {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.loadVectorSnappingData');

    var this_ = this;
    var oBo = null;
    this_.oSnapOverlayLayer_.setVisible(this_['snapOptions']['visible']);
    this_.oSnapOverlayLayer_.getSource().clear();
    this_.checkEditionScale();
    oVmap.getMap().getOLMap().removeInteraction(this_.snap_);
    setTimeout(function () {
        if (this_['currentAction'] === 'basicTools-select-editFeature') {
            if (this_['isMaxScaleOk'] && this_['isMinScaleOk']) {
                for (var i = 0; i < this_['aQueryableBOs'].length; i++) {
                    oBo = this_['aQueryableBOs'][i];
                    this_.stopLoadingBoVectorSnappingData(oBo);
                    this_.loadBoVectorSnappingData(oBo);
                }
                for (var i = 0; i < oBo.length; i++) {
                    if (oBo[i]['bo_snapping_loaded'] === null) {
                        oBo[i]['bo_snapping_enabled'] = false;
                    }
                }
                oVmap.getMap().getOLMap().addInteraction(this_.snap_);
            }
        }
    });
};

/**
 * Retrieve the object given in parameter with ajaxRequest
 * @param {type} oBo
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.loadBoVectorSnappingData = function (oBo) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.loadBoVectorSnappingData');
    var this_ = this;
    var bo_id = oBo['bo_id'];
    var aFeatures = [];
    var geom = null;
    var extent = oVmap.getMap().getOLMap().getView().calculateExtent(oVmap.getMap().getOLMap().getSize());
    var polygonGeom = new ol.geom.Polygon.fromExtent(extent);
    if (oBo['bo_snapping_enabled'] === true) {
        oBo['bo_snapping_loaded'] = false;
        ajaxRequest({
            'method': 'POST',
            'url': oVmap['properties']['api_url'] + '/vmap/querys/' + bo_id + '/geometry',
            'headers': {
                'X-HTTP-Method-Override': 'GET',
                'Accept': 'application/x-vm-json'
            },
            'data': {
                'intersect_geom': oVmap.getEWKTFromGeom(polygonGeom),
                'snapping_limit': this_['snapOptions']['snappingObjectsLimit'],
                'snapping_mode': this_['snapOptions']['mode']
            },
            'ajaxLoader': false,
            'abord': oBo['loadCanceller'].promise,
            'scope': this.$scope_,
            'success': function (response) {
                if (!goog.isDefAndNotNull(response['data'])) {
                    oBo['bo_snapping_loaded'] = null;
                    return 0;
                }
                if (goog.isDefAndNotNull(response['data']['errorMessage'])) {
                    console.error(response['data']['errorMessage']);
                    oBo['bo_snapping_loaded'] = null;
                    return 0;
                }
                if (!goog.isDefAndNotNull(response['data'][0])) {
                    oBo['bo_snapping_loaded'] = null;
                    return 0;
                }
                if (response['data'][0]['count'] > this_['snapOptions']['snappingObjectsLimit']) {
                    oBo['bo_snapping_loaded'] = null;
                    var text = 'Limit de points atteinte pour object ';
                    text += oBo['bo_title'];
                    $.notify(text, "warn");
                    return 0;
                }
                oBo['bo_snapping_loaded'] = true;
                for (var i = 0; i < response['data'].length; i++) {
                    if (goog.isDefAndNotNull(response['data'][i]['geom'])) {
                        geom = response['data'][i]['geom'];
                        aFeatures.push(new ol.Feature({
                            geometry: oVmap.getGeomFromEWKT(geom),
                            'bo_id': bo_id
                        }));
                    }
                }
                this_.oSnapOverlayLayer_.getSource().addFeatures(aFeatures);
            },
            'error': function (response) {
                oBo['bo_snapping_loaded'] = null;
            }
        });
    }
};

/**
 * Cancel request with a resolve callback
 * @param {type} oBo
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.stopLoadingBoVectorSnappingData = function (oBo) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.stopLoadingBoVectorSnappingData');
    var this_ = this;
    if (goog.isDefAndNotNull(oBo['loadCanceller'])) {
        oBo['loadCanceller'].resolve();
        oBo['loadCanceller'] = this_.$q_.defer();
        oBo['bo_snapping_loaded'] = null;
    }
};


// GetFeatureInfo

/**
 * Clear the GetFeatureInfo result
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.clearGetFeatureInfoResult = function () {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.clearGetFeatureInfoResult');

    goog.array.clear(this['aGetFeatureInfoResult']);
    this['iGetFeatureInfoWithResultLayers'] = 0;
    this['iGetFeatureInfoWithoutResultLayers'] = 0;
    this['aGetFeatureInfoSelectedLayer'] = 0;
    this['aGetFeatureInfoSelectedSubLayer'] = 0;
};

/**
 * Show the GetFeatureInfo result window
 * @param {array} aGetFeatureInfoTree
 * @param {array} aCoordinate
 * @param  {boolean} bModalOpened Détermine si la modale GetFeatureInfo est déjà ouverte
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.showGetFeatureInfoResultWindow = function (aGetFeatureInfoResult, aCoordinate, bModalOpened) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.showGetFeatureInfoResultWindow');

    var this_ = this;
    var bHaveResult = false;
    var selectedLayer = null;
    var selectedSubLayer = null;
    var iResultedLayers = 0;
    var iResultedSublayers = 0;
    var iNoResultedLayers = 0;
    var bModalOpened = goog.isDefAndNotNull(bModalOpened) ? bModalOpened : false;

    // Si la modale est déjà ouverte, sauvegarde le couple calque/couche sélectionné
    if (bModalOpened) {
        selectedLayer = angular.copy(this['aGetFeatureInfoSelectedLayer']);
        selectedSubLayer = angular.copy(this['aGetFeatureInfoSelectedSubLayer']);
    }

    // Nettoie les variables d'environnement
    this.clearGetFeatureInfoResult();

    if (!goog.isArray(aGetFeatureInfoResult)) {
        return 0;
    }

    // Remplit bHaveResult
    for (var i = 0; i < aGetFeatureInfoResult.length; i++) {
        if (goog.isArray(aGetFeatureInfoResult[i]['aSubLayers'])) {
            for (var ii = 0; ii < aGetFeatureInfoResult[i]['aSubLayers'].length; ii++) {
                if (goog.isDefAndNotNull(aGetFeatureInfoResult[i]['aSubLayers'][ii]['result'])) {
                    bHaveResult = true;
                }
            }
        }
    }
    if (!bHaveResult) {
        $.notify('Aucun résultat disponible');
        if (!bModalOpened) {
            return 0;
        }
    }

    // Ordonne les calques et couches par nom
    goog.array.sortObjectsByKey(aGetFeatureInfoResult, 'name');
    for (var i = 0; i < aGetFeatureInfoResult.length; i++) {
        if (goog.isArray(aGetFeatureInfoResult[i]['aSubLayers'])) {
            aGetFeatureInfoResult[i]['aSubLayers'].sort(function(a, b){
                var nameA=a['name'].toLowerCase(), nameB=b['name'].toLowerCase();
                if (nameA < nameB)
                    return -1;
                if (nameA > nameB)
                    return 1;
                return 0;
            });
            for (var ii = 0; ii < aGetFeatureInfoResult[i]['aSubLayers'].length; ii++) {
                aGetFeatureInfoResult[i]['aSubLayers'][ii]['index'] = angular.copy(ii);
            }
        }
    }

    // Choisit le calque/couche à choisir en premier et compte les calques ayant des résultats
    for (var i = 0; i < aGetFeatureInfoResult.length; i++) {
        if (goog.isArray(aGetFeatureInfoResult[i]['aSubLayers'])) {
            for (var ii = 0; ii < aGetFeatureInfoResult[i]['aSubLayers'].length; ii++) {
                if (goog.isDefAndNotNull(aGetFeatureInfoResult[i]['aSubLayers'][ii]['result'])) {
                    if (!goog.isDefAndNotNull(selectedLayer) && !goog.isDefAndNotNull(selectedSubLayer)) {
                        selectedLayer = angular.copy(i);
                        selectedSubLayer = angular.copy(ii);
                    }
                }
            }
        }
    }

    // Rempli iResultedLayers et iNoResultedLayers
    for (var i = 0; i < aGetFeatureInfoResult.length; i++) {
        iResultedSublayers = 0;
        if (goog.isArray(aGetFeatureInfoResult[i]['aSubLayers'])) {
            for (var ii = 0; ii < aGetFeatureInfoResult[i]['aSubLayers'].length; ii++) {
                if (goog.isDefAndNotNull(aGetFeatureInfoResult[i]['aSubLayers'][ii]['result'])) {
                    iResultedSublayers++;
                }
            }
        }
        if (iResultedSublayers > 0) {
            iResultedLayers++;
        } else {
            iNoResultedLayers++;
        }
    }

    // Fait confiance aux URL
    for (var i = 0; i < aGetFeatureInfoResult.length; i++) {
        if (goog.isArray(aGetFeatureInfoResult[i]['aSubLayers'])) {
            for (var ii = 0; ii < aGetFeatureInfoResult[i]['aSubLayers'].length; ii++) {
                if (goog.isDefAndNotNull(aGetFeatureInfoResult[i]['aSubLayers'][ii]['url'])) {
                    aGetFeatureInfoResult[i]['aSubLayers'][ii]['url_trusted'] = this.$sce_.trustAsResourceUrl(aGetFeatureInfoResult[i]['aSubLayers'][ii]['url']);
                }
            }
        }
    }

    // Sauvegarde les nouvelles données sur le controler
    this.$scope_.$applyAsync(function(){
        this_.$scope_['collapseSublayersTabs'] = false;
        this_['aGetFeatureInfoResult'] = aGetFeatureInfoResult;
        this_['aGetFeatureInfoSelectedLayer'] = goog.isDefAndNotNull(selectedLayer) ? selectedLayer : 0;
        this_['aGetFeatureInfoSelectedSubLayer'] = goog.isDefAndNotNull(selectedSubLayer) ? selectedSubLayer : 0;
        this_['iGetFeatureInfoWithResultLayers'] = goog.isDefAndNotNull(iResultedLayers) ? iResultedLayers : 0;
        this_['iGetFeatureInfoWithoutResultLayers'] = goog.isDefAndNotNull(iNoResultedLayers) ? iNoResultedLayers : 0;
    })

    // Ajoute les coordonnées sur la carte
    this.oGetFeatureInfoOverlay_.getSource().addFeature(new ol.Feature({
        geometry: new ol.geom.Point(aCoordinate)
    }));

    $('#getFeatureInfo-result-window').show();

    this.setCollapseSublayersTabs();
};


/**
 * Set scope.collapseSublayersTabs
 *
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.setCollapseSublayersTabs = function () {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.setCollapseSublayersTabs');

    var this_ = this;

    this_.$scope_['collapseSublayersTabs'] = false;
    setTimeout(function () {
        // Permet de rejoindre les onglets si il y en a beaucoup
        this_.$scope_.$applyAsync(function(){
            this_.$scope_['collapseSublayersTabs'] = $('#select-getFeatureInfo-result-window-sublayer-tabs').innerHeight() > 50;
        });
    });
};

/**
 * Close the GetFeatureInfo result window
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.closeGetFeatureInfoResultWindow = function () {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.closeGetFeatureInfoResultWindow');

    // Cache la modale
    $('#getFeatureInfo-result-window').hide();

    // Supprime les points GetFeatureInfo
    this.oGetFeatureInfoOverlay_.getSource().clear();
};

/**
 * Clone the form
 * @param {object} selection
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.cloneForm = function (selection) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.cloneForm');

    // Ferme le formulaire d'édition
    $('#select-edit-modal').modal('hide');
    $('#select-list-modal').modal('hide');

    // Affiche l'outil d'insertion
    oVmap.getToolsManager().getBasicTools().toggleTool($('#basic-tools-dropdown-insert-btn'));

    var insertScope = angular.element($('#vmap-insert-tool')).scope();
    insertScope['ctrl']['displayEditFromToClone'](selection, this.oFormValuesUpdate);
};

/**
 * Open edit form after insert
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.displayEditFromAfterInsert = function (sBOID, response) {
    oVmap.log('nsVmap.nsToolsManager.Select.prototype.selectController.prototype.displayEditFromAfterInsert');

    var this_ = this;
    var basicSelectScope = angular.element($('#vmap-basicselect-tool')).scope();
    var oBusinessObject = basicSelectScope['ctrl']['oQueryableBOs'][sBOID];

    if (goog.isDefAndNotNull(oBusinessObject)) {

        var boIdValue = response['data'][oBusinessObject['bo_id_field']];

        // Si la fênetre d'édition doit s'ouvrir après l'ajout
        if (oBusinessObject['bo_enable_automatically_edit'] && oBusinessObject['bo_user_rights'].indexOf('UPDATE') !== -1) {

            var data = {
                bo_id_value: boIdValue,
                bo_type: oBusinessObject['bo_id'],
                bo_title: oBusinessObject['bo_title'],
                bo_id_field: oBusinessObject['bo_id_field'],
                geom_column: oBusinessObject['bo_geom_column'],
                enable_geom_updater: oBusinessObject['enable_geom_updater'],
                have_update_rights: oBusinessObject['bo_user_rights'].indexOf('UPDATE') !== -1,
                have_card_rights: oBusinessObject['bo_user_rights'].indexOf('SELECT') !== -1,
                have_delete_rights: oBusinessObject['bo_user_rights'].indexOf('DELETE') !== -1,
                have_insert_rights: oBusinessObject['bo_user_rights'].indexOf('INSERT') !== -1,
            }

            this_.displayEditFrom(data);
        }
    }
};

/**
 * Open modal to add geometry
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.openFormForInsertGeom = function () {
    oVmap.log('nsVmap.nsToolsManager.Insert.inserttoolControlle.openFormForInsertGeom');

    this.resetInteractionAndTooltipAndSnap();

    this.resetGeomValueFormForInsertGeom();

    // Affiche le formulaire
    $('#vmap-select-insert-geometry-modal').modal('show');
};

/**
 * Close modal to add geometry
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.closeFormForInsertGeom = function () {
    oVmap.log('nsVmap.nsToolsManager.Insert.inserttoolControlle.closeFormForInsertGeom');

    this.resetInteractionAndTooltipAndSnap();

    // Affiche le formulaire
    $('#vmap-select-insert-geometry-modal').modal('hide');
};

/**
 * Insert geometry in oOverlayLayer_
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.insertGeometry = function () {
    oVmap.log('nsVmap.nsToolsManager.Insert.inserttoolControlle.insertGeometry');

    if (!goog.isDefAndNotNull(this['selectTypeInsertGeom']) || this['selectTypeInsertGeom'] == '') {
        $.notify("Le type de géométrie est obligatoire", "error");
        return;
    }

    if (!goog.isDefAndNotNull(this['addGeomProj']) || this['addGeomProj'] == '') {
        $.notify("La projection est obligatoire", "error");
        return;
    }
    var aFeatures;
    var currentProjection = oVmap.getMap().getOLMap().getView().getProjection().getCode();
    if (this['selectTypeInsertGeom'] == 'XY') {
        if (!goog.isDefAndNotNull(this['addGeomPointX']) || this['addGeomPointX'] == '' ||
            !goog.isDefAndNotNull(this['addGeomPointY']) || this['addGeomPointY'] == '') {
            $.notify("Les coordonnées X et Y sont requis", "error");
            return;
        }

        sCoordX = this['addGeomPointX'].replace(',', '.');
        sCoordY = this['addGeomPointY'].replace(',', '.');

        var coordinates = [parseFloat(sCoordX), parseFloat(sCoordY)];
        var projectedCoordinates = ol.proj.transform(coordinates, this['addGeomProj'], currentProjection);

        aFeatures = [new ol.Feature(new ol.geom.Point(projectedCoordinates))];

        if (isNaN(projectedCoordinates[0]) || isNaN(projectedCoordinates[1])) {
            bootbox.alert('<h4>Ces coordonnées ne sont pas disponibles dans la projection saisie</h4>');
            return 0;
        }

    } else {
        if (!goog.isDefAndNotNull(this['addGeomWKT']) || this['addGeomWKT'] == '') {
            $.notify("La géométrie est requise", "error");
            return;
        }

        var format = new ol.format.WKT();

        var oFeature = format.readFeature(this['addGeomWKT'], {
            dataProjection: this['addGeomProj'],
            featureProjection: currentProjection,
        });

        if (!oVmap.getMapManager().isAuthorizedGeomType(this.editableFeatureType, oFeature.getGeometry().getType())) {
            $.notify("La géométrie ne correspond pas au type de géométrie accepté par cet objet.", "error");
            return;
        }

        aFeatures = oVmap.getMapManager().cutTheFeature(oFeature);
    }

    var this_ = this;

    this.oOverlayLayer_.getSource().addFeatures(aFeatures);
    // Enregistre les changements
    this_.putFeaturesOnTheElement(this_.oOverlayFeatures_.getArray());

    var extent = ol.extent.createEmpty();
    for (var i = 0; i < aFeatures.length; i++) {
        ol.extent.extend(extent, aFeatures[i].getGeometry().getExtent());
    }
    this['map'].getView().fit(extent, {
        size: this['map'].getSize(),
        maxZoom: oVmap['properties']['vmap']['max_zoom']
    });

    this.closeFormForInsertGeom();
};

/**
 * Reset form values on modal
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.resetValuesFormForInsertGeom = function () {
    oVmap.log('nsVmap.nsToolsManager.Insert.inserttoolControlle.resetValuesFormForInsertGeom');

    this['addGeomPointX'] = '';
    this['addGeomPointY'] = '';

    this.resetGeomValueFormForInsertGeom();

    this['selectTypeInsertGeom'] = 'WKT';
    this['onlySelectWKT'] = false;
};

/**
 * Reset geom value on modal
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.resetGeomValueFormForInsertGeom = function () {
    oVmap.log('nsVmap.nsToolsManager.Insert.inserttoolControlle.resetGeomValueFormForInsertGeom');
    var geom = '';
    
    if (goog.isDefAndNotNull(this['editableSelection']['olFeature'])
        && goog.isDefAndNotNull(this['editableSelection']['olFeature'].getGeometry())) {

        // Si on met à jour la géométrie on met la bonne valeur pour la projection
        this['addGeomProj'] = oVmap.getMap().getOLMap().getView().getProjection().getCode();
        var featureGeom = this['editableSelection']['olFeature'].getGeometry();
        var format = new ol.format.WKT();
        geom = format.writeGeometry(featureGeom);

        // Si c'est un point on saisie aussi les coordonnées x & y
        if (this['editableSelection']['olFeature'].getGeometry().getType() === 'Point') {
            var coord = this['editableSelection']['olFeature'].getGeometry().getCoordinates();
            this['addGeomPointX'] = coord[0];
            this['addGeomPointY'] = coord[1];
        }
    }

    this['addGeomWKT'] = geom;
};

/**
 * 
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.isAuthorizedGeomType = function (sGeomTested) {
    oVmap.log('nsVmap.nsToolsManager.Insert.inserttoolControlle.isAuthorizedGeomType');
    if (goog.isDefAndNotNull(this.editableFeatureType)) {
        return oVmap.getMapManager().isAuthorizedGeomType(this.editableFeatureType, sGeomTested);
    }
};

/**
 * Reset interractions, hide tooltip and delete snapping
 * @export
 */
nsVmap.nsToolsManager.Select.prototype.selectController.prototype.resetInteractionAndTooltipAndSnap = function () {
    oVmap.log('nsVmap.nsToolsManager.Insert.inserttoolControlle.resetInteractionAndTooltipAndSnap');
    // Supprime les interractions
    oVmap.getMap().removeActionsOnMap();

    // Cache la tooltip
    oVmap.getMap().getMapTooltip().hide();

    // Supprime le snapping
    this.oSnapOverlayLayer_.getSource().clear();
    oVmap.getMap().getOLMap().removeInteraction(this.snap_);
};

// Définit la directive et le controller
oVmap.module.directive('appSelect', nsVmap.nsToolsManager.Select.prototype.selectDirective);
oVmap.module.controller('AppSelectController', nsVmap.nsToolsManager.Select.prototype.selectController);
