/* global oVmap, nsVmap, goog, ol, vitisApp */

/**
 * @author: Anthony Borghi
 * @Description: Fichier contenant la classe nsVmap.nsToolsManager.Urlexporter
 * cette classe permet à l'utilisateur d'exporter une url pour un autre utilisateur
 */
goog.provide('nsVmap.nsToolsManager.Urlexporter');

goog.require('oVmap');

/**
 * @classdesc
 * Class {@link nsVmap.nsToolsManager.VmapUser}
 * @constructor
 * @export
 */
nsVmap.nsToolsManager.Urlexporter = function () {
    oVmap.log('nsVmap.nsToolsManager.Urlexporter');
};



/**
 * Directive
 * @return {angular.Directive} The directive specs.
 * @constructor
 */
nsVmap.nsToolsManager.Urlexporter.prototype.urlExporterDirective = function () {
    oVmap.log("nsVmap.nsToolsManager.Urlexporter.prototype.urlExporterDirective");
    return {
        restrict: 'A',
        scope: {
            'map': '=appMap',
            'lang': '=appLang'
        },
        controller: 'AppUrlExporterController',
        controllerAs: 'ctrl',
        bindToController: true,
        templateUrl: oVmap['properties']['vmap_folder'] + '/' + 'template/tools/' + (oVmap['properties']['is_mobile'] ? 'urlexporter_mobile.html' : 'urlexporter.html')
    };
};

/**
 * Controler
 * @constructor
 * @param {object} $scope
 * @returns {undefined}
 * @ngInject
 */
nsVmap.nsToolsManager.Urlexporter.prototype.urlExporterController = function ($scope, $q, $element) {
    oVmap.log("nsVmap.nsToolsManager.Urlexporter.prototype.urlExporterController");

    var this_ = this;

    this.$scope_ = $scope;

    /**
     * The current properties
     */
    this['properties'] = oVmap['properties'];

    /**
     * The current token
     */
    this['token'] = oVmap['properties']['token'];

    /**
     * @type {string}
     */
    this["privateExportUrl"] = "";

    /**
     * @type {string}
     */
    this["publicExportUrl"] = "";

    // recalcule l'url quand on change de carte
    oVmap['scope'].$on('mapChanged', function () {
        this_["privateExportUrl"] = this_.getExportUrl();
        this_["publicExportUrl"] = this_.getExportUrl(true);
    });

    // recalcule l'url quand on bouge la  carte
    oVmap.getMap().getOLMap().on("moveend", function(event){
        this_["privateExportUrl"] = this_.getExportUrl();
        this_["publicExportUrl"] = this_.getExportUrl(true);
    });

    // Recalcule l'URL
    this_["privateExportUrl"] = this_.getExportUrl();
    this_["publicExportUrl"] = this_.getExportUrl(true);
};

/**
 * Generates and return the export URL
 * @param  {boolean} bPublic
 * @return {string}
 * @export
 */
nsVmap.nsToolsManager.Urlexporter.prototype.urlExporterController.prototype.getExportUrl = function (bPublic) {
    oVmap.log("nsVmap.nsToolsManager.Urlexporter.urlExporterController.getExportUrl");

    var sUrl = this['properties']['web_server_name'] + "/" + this['properties']['application'];

    if (goog.isDefAndNotNull(this['properties']['environment'])){
        if (this['properties']['environment'] != ""){
            sUrl += "_" + this['properties']['environment'];
        }
    }

    sUrl += "?mode_id=vmap&";

    var oMapCatalog = oVmap.getMapManager().getMapCatalog();
    var iMapId = oMapCatalog["maps"][oMapCatalog["usedMap"]]["map_id"];

    sUrl += "map_id=" + iMapId;

    // si public add token
    if (bPublic === true){
        sUrl += "&token=" + this['properties']['public_token'];
    }

    // get extent and add in url
    var sExtent = oVmap.getMap().getOLMap().getView().calculateExtent(oVmap.getMap().getOLMap().getSize()).join("|");
    sUrl += "&extent=" + encodeURI(sExtent);

    return sUrl;
};

/**
 * Copy the generated URL on the clipboard
 * @param  {boolean} bPublic
 * @export
 */
nsVmap.nsToolsManager.Urlexporter.prototype.urlExporterController.prototype.copyUrl = function (bPublic) {
    oVmap.log("nsVmap.nsToolsManager.Urlexporter.urlExporterController.copyUrl");

    /* Get the text field */
    if (bPublic === true) {
        var copyText = document.getElementById("publicExportUrlField");
    } else {
        var copyText = document.getElementById("privateExportUrlField");
    }

    /* Select the text field */
    copyText.select();

    /* Copy the text inside the text field */
    document.execCommand("copy");

    $.notify('Copié', 'success');
}

// Définit la directive et le controller
oVmap.module.directive('appUrlExporter', nsVmap.nsToolsManager.Urlexporter.prototype.urlExporterDirective);
oVmap.module.controller('AppUrlExporterController', nsVmap.nsToolsManager.Urlexporter.prototype.urlExporterController);
